package com.psp.payment.common.facades.order.impl;

import com.psp.data.PspCardAuthorizationData;
import com.psp.enums.PspTransactionStatus;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.payment.common.dto.PspBrand;
import com.psp.payment.common.dto.PspPaymentModeData;
import com.psp.payment.common.enums.PspAuthResult;
import com.psp.payment.common.facades.order.PspCheckoutFacade;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.services.PspCustomerAccountService;
import com.psp.payment.common.services.PspPaymentModeService;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import com.psp.payment.flexpayments.strategies.PspFlexPaymentInfoCreateStrategy;
import com.psp.payment.flexpayments.strategies.impl.PspCreateFlexPaymentRequestStrategyImpl;
import de.hybris.platform.acceleratorfacades.order.impl.DefaultAcceleratorCheckoutFacade;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNullStandardMessage;

public class PspCheckoutFacadeImpl extends DefaultAcceleratorCheckoutFacade implements PspCheckoutFacade {

    private static final Logger LOG = Logger.getLogger(PspCheckoutFacadeImpl.class);

    @Resource
    private PspUtils pspUtils;

    @Resource
    private PspCreateFlexPaymentRequestStrategyImpl pspCreateFlexPaymentRequestStrategy;

    @Resource
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Resource
    private Converter<PspFlexPaymentRequest, PaymentData> pspFlexPaymentDataConverter;

    @Resource
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Resource
    private PspPaymentModeService paymentModeService;

    @Resource
    private PspCustomerAccountService customerAccountService;

    @Override
    public PspCardAuthorizationData authorizeCardPayment(String securityCode) {
        PspCardAuthorizationData authorization = new PspCardAuthorizationData();
        authorization.setResult(PspAuthResult.FAILURE);
        CartModel cartModel = getCart();

        if (validateCurrentUser()) {
            String subScriptionId = getSubscriptionId(cartModel);

            if (StringUtils.isNotBlank(subScriptionId)) {
                PaymentTransactionEntryModel paymentTransactionEntryModel = authorize(securityCode, cartModel);

                if (paymentTransactionEntryModel != null) {
                    if (PspTransactionStatus.UNCERTAIN.name().equals(paymentTransactionEntryModel.getTransactionStatus())) {
                        authorization.setResult(PspAuthResult.UNCERTAIN);
                    } else if (TransactionStatus.ACCEPTED.name().equals(paymentTransactionEntryModel.getTransactionStatus())
                            || TransactionStatus.REVIEW.name().equals(paymentTransactionEntryModel.getTransactionStatus())) {
                        if (!paymentTransactionEntryModel.getTransactionStatusDetails().equals(TransactionStatusDetails.THREE_D_SECURE_AUTHENTICATION_REQUIRED.name())) {
                            authorization.setResult(PspAuthResult.SUCCESS);
                        } else {
                            String decodedHtml = pspUtils.decodeUsingBase64Encryption(cartModel.getPspIntegrationResult());
                            authorization.setHtml(decodedHtml);
                            authorization.setResult(PspAuthResult.THREEDSECURE);

                            // Clean up transaction and cart
                            Collection<Object> paymentTransactionAndEntry = Arrays.asList(paymentTransactionEntryModel.getPaymentTransaction(), paymentTransactionEntryModel);
                            getModelService().removeAll(paymentTransactionAndEntry);
                            cartModel.setPspIntegrationResult(StringUtils.EMPTY);
                            getModelService().save(cartModel);
                        }
                    }
                }
            }
        }

        return authorization;
    }


    @Override
    public PaymentData createFlexPaymentData() {
        CartModel cart = getCart();
        PspFlexPaymentRequest pspFlexPaymentRequest = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cart);
        return pspFlexPaymentDataConverter.convert(pspFlexPaymentRequest);
    }

    @Override
    protected CCPaymentInfoData getPaymentDetails() {
        final CartModel cart = getCart();
        if (cart != null) {
            final PaymentInfoModel paymentInfo = cart.getPaymentInfo();
            if (paymentInfo != null) {
                return pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(paymentInfo).convert(paymentInfo);
            }
        }

        return null;
    }

    @Override
    public void saveSubscription(PspPaymentModeData pspPaymentModeData) {
        PspBrand pspBrand = pspPaymentModeData.getPaymentModes().get(0).getBrands().get(0);
        ComposedTypeModel composedTypeModel = pspBrand.getComposedTypeModel();
        PspFlexPaymentInfoCreateStrategy pspFlexPaymentInfoCreateStrategy = (PspFlexPaymentInfoCreateStrategy) pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel);
        pspFlexPaymentInfoCreateStrategy.saveSubscription(getCart());
    }

    @Override
    public void saveBillingAddressToPaymentInfo() {
        CartModel cartModel = getCart();
        AddressModel billingAddress = cartModel.getPaymentAddress();

        if (billingAddress != null) {
            PaymentInfoModel paymentInfoModel = cartModel.getPaymentInfo();
            AddressModel existingAssociatedAddress = paymentInfoModel.getBillingAddress();

            if (existingAssociatedAddress == null) {
                AddressModel newBillingAddress = getModelService().clone(billingAddress);
                newBillingAddress.setOwner(paymentInfoModel);
                paymentInfoModel.setBillingAddress(newBillingAddress);
                getModelService().save(newBillingAddress);
                getModelService().save(paymentInfoModel);
            }
        }
    }

    @Override
    public void setPaymentModeIfAvailable() {
        setPaymentMode(paymentModeService.getAvailablePaymentMethods());
    }

    @Override
    public void setPaymentMode(String code) {
        setPaymentMode(paymentModeService.getAvailablePaymentModeForCode(code));
    }

    @Override
    public String fetchPaymentErrorCode() {
        CartModel cartModel = getCart();
        String errorCode = cartModel.getPaymentErrorCode();

        if (StringUtils.isEmpty(errorCode)) {
            errorCode = cartModel.getPaymentErrorStatus();
        }

        cartModel.setPaymentErrorCode(StringUtils.EMPTY);
        cartModel.setPaymentErrorStatus(StringUtils.EMPTY);
        getModelService().save(cartModel);

        return errorCode;
    }

    @Override
    public boolean setPaymentDetails(String paymentInfoId) {
        validateParameterNotNullStandardMessage("paymentInfoId", paymentInfoId);

        if (checkIfCurrentUserIsTheCartUser() && StringUtils.isNotBlank(paymentInfoId)) {
            final CustomerModel currentUserForCheckout = getCurrentUserForCheckout();
            final PaymentInfoModel paymentInfoModel = customerAccountService.getPaymentInfoForCode(currentUserForCheckout, paymentInfoId);
            final CartModel cart = getCart();
            if (paymentInfoModel != null) {
                final CommerceCheckoutParameter parameter = new CommerceCheckoutParameter();
                parameter.setEnableHooks(true);
                parameter.setCart(cart);
                parameter.setPaymentInfo(paymentInfoModel);
                return getCommerceCheckoutService().setPaymentInfo(parameter);
            } else {
                LOG.warn(String
                        .format(
                                "Did not find CreditCardPaymentInfoModel for user: %s, cart: %s &  paymentInfoId: %s. PaymentInfo Will not get set.",
                                currentUserForCheckout, cart, paymentInfoId));
            }
        }

        return false;
    }

    protected PaymentTransactionEntryModel authorize(String securityCode, CartModel cartModel) {
        final CommerceCheckoutParameter parameter = new CommerceCheckoutParameter();
        parameter.setEnableHooks(true);
        parameter.setCart(cartModel);
        parameter.setSecurityCode(securityCode);
        parameter.setPaymentProvider(getPaymentProvider());

        return getCommerceCheckoutService().authorizePayment(parameter);
    }

    protected boolean validateCurrentUser() {
        return checkIfCurrentUserIsTheCartUser();
    }

    private String getSubscriptionId(CartModel cartModel) {
        String subScriptionId = null;
        PaymentInfoModel paymentInfoModel = cartModel.getPaymentInfo();

        if (paymentInfoModel instanceof CreditCardPaymentInfoModel) {
            CreditCardPaymentInfoModel creditCardPaymentInfoModel = (CreditCardPaymentInfoModel) paymentInfoModel;
            subScriptionId = creditCardPaymentInfoModel.getSubscriptionId();
        } else if (paymentInfoModel instanceof DirectDebitPaymentInfoModel) {
            DirectDebitPaymentInfoModel directDebitPaymentInfoModel = (DirectDebitPaymentInfoModel) paymentInfoModel;
            subScriptionId = directDebitPaymentInfoModel.getSubscriptionId();
        }
        return subScriptionId;
    }

    private void setPaymentMode(List<PaymentModeModel> availablePaymentModes) {
        CartModel cartModel = getCart();

        if (cartModel != null && CollectionUtils.isNotEmpty(availablePaymentModes)) {
            cartModel.setPaymentMode(availablePaymentModes.get(0));
            getModelService().save(cartModel);
        }
    }
}
