package com.psp.payment.common.services.impl;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.model.AbstractFlexPaymentInfoModel;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.services.PspPaymentService;
import com.psp.payment.common.strategies.PspPostPaymentFeedbackStrategy;
import com.psp.payment.common.strategies.PspSignatureValidationStrategy;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.dto.PspAuthorizationResult;
import com.psp.payment.directlink.dto.PspSubscriptionAuthorizationRequest;
import com.psp.payment.flexpayments.strategies.PspFlexPaymentInfoCreateStrategy;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import com.psp.payment.tokenisation.factories.PspResponseStrategyFactory;
import com.psp.payment.tokenisation.strategies.PspCreateSubscriptionRequestStrategy;
import com.psp.payment.tokenisation.strategies.PspPaymentResponseInterpretationStrategy;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.acceleratorservices.payment.data.PaymentSubscriptionResultItem;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.commands.request.AuthorizationRequest;
import de.hybris.platform.payment.commands.result.AuthorizationResult;
import de.hybris.platform.payment.dto.BillingInfo;
import de.hybris.platform.payment.dto.CardInfo;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.impl.DefaultPaymentServiceImpl;
import de.hybris.platform.payment.methods.CardPaymentService;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/**
 * PSP specific implementation of hybris payment service.
 */
public class PspPaymentServiceImpl extends DefaultPaymentServiceImpl implements PspPaymentService {
    private static final Logger LOG = LoggerFactory.getLogger(PspPaymentServiceImpl.class);

    private static final String FUNCTIONALITY_UNSUPPORTED = "This functionality is not supported.";

    @Resource
    private PspCreateSubscriptionRequestStrategy pspCreateSubscriptionRequestStrategy;

    @Resource
    private Converter<PspCreateSubscriptionRequest, PaymentData> pspPaymentDataConverter;

    @Resource
    private CartService cartService;

    @Resource
    private CardPaymentService cardPaymentService;

    @Resource
    private PspPaymentResponseInterpretationStrategy pspPaymentResponseInterpretationStrategy;

    @Resource
    private PspUtils pspUtils;

    @Resource
    private PspResponseStrategyFactory pspResponseStrategyFactory;

    @Resource
    private PspSignatureValidationStrategy pspSignatureValidationStrategy;

    @Resource
    private CommonI18NService commonI18NService;

    @Resource
    private ModelService modelService;

    @Resource
    private PspPostPaymentFeedbackStrategy pspPostPaymentFeedbackStrategy;

    @Resource
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Override
    public PaymentData beginHopCreatePaymentSubscription(final String siteName, final String responseUrl,
                                                         final String merchantCallbackUrl, final CustomerModel customer, final CreditCardPaymentInfoModel cardInfo,
                                                         final AddressModel paymentAddress) {

        CartModel cartModel = cartService.getSessionCart();
        final PspCreateSubscriptionRequest request = pspCreateSubscriptionRequestStrategy
                .createSubscriptionRequest(cartModel, customer);
        PaymentData data = pspPaymentDataConverter.convert(request);

        if (data == null) {
            data = new PaymentData();
            data.setParameters(new HashMap<>());
        }

        return data;
    }

    @Override
    public PaymentData beginSopCreatePaymentSubscription(String siteName, String responseUrl,
                                                         String merchantCallbackUrl, CustomerModel customerModel, CreditCardPaymentInfoModel cardInfo,
                                                         AddressModel paymentAddress) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public PaymentSubscriptionResultItem completeHopCreatePaymentSubscription(CustomerModel customerModel,
                                                                              boolean saveInAccount, Map<String, String> parameters) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public PaymentSubscriptionResultItem completeSopCreatePaymentSubscription(CustomerModel customerModel,
                                                                              boolean saveInAccount, Map<String, String> parameters) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public void handleCreateSubscriptionCallback(Map<String, String> parameters) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public void handleFraudUpdateCallback(Map<String, String> parameters) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public void setPaymentTransactionReviewResult(PaymentTransactionEntryModel reviewDecisionEntry, String orderCode) {
        throw new UnsupportedOperationException(FUNCTIONALITY_UNSUPPORTED);
    }

    @Override
    public PspPaymentSubscriptionResultItem completeHopCreatePaymentSubscription(Map<String, String> parameters) {
        validateParameterNotNull(parameters, "Map<String, String>  cannot be null");

        if (pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)) {
            LOG.debug("PSP signature validated, validating the current cart.");
            boolean isResponseValidForCurrentCart = validateCartDetails(parameters, PspConstants.KEY_RESULT_ORDERID, null);

            if (isResponseValidForCurrentCart) {
                LOG.debug("Session cart code same as one received in the response parameters.");
                final PspCreateSubscriptionResult response = pspPaymentResponseInterpretationStrategy
                        .interpretResponse(parameters);
                return pspResponseStrategyFactory.getResponseStrategy(response).handleResponse(response);
            }
        }

        return null;
    }

    @Override
    public boolean validateCartDetails(Map<String, String> parameters, String orderIdKey, String amountKey) {
        boolean isResponseValid = false;

        if (MapUtils.isNotEmpty(parameters) && StringUtils.isNotEmpty(orderIdKey)) {
            CartModel cartModel = cartService.getSessionCart();

            if (cartModel != null) {
                // Cart code
                String sessionCartCode = cartModel.getCode();
                String orderNumberParameterValue = parameters.get(orderIdKey);
                String cartNumberFromResponse = StringUtils.substringBefore(orderNumberParameterValue, PspConstants.HYPHEN);
                isResponseValid = StringUtils.equals(sessionCartCode, cartNumberFromResponse);
            }

            // Cart amount
            if (isResponseValid && StringUtils.isNotEmpty(amountKey)) {
                Double totalPriceFromCart = cartModel.getTotalPrice();
                String amountParameterValue = parameters.get(amountKey);
                Double amountFromResponse = 0.0;

                if (NumberUtils.isNumber(amountParameterValue)) {
                    amountFromResponse = NumberUtils.createDouble(amountParameterValue);
                }

                isResponseValid = totalPriceFromCart.compareTo(amountFromResponse) == 0;
            }
        }

        return isResponseValid;
    }

    @Override
    public PaymentTransactionEntryModel authorize(String merchantTransactionCode, BigDecimal amount, Currency currency,
                                                  AddressModel deliveryAddress, String subscriptionID, String cv2,
                                                  String paymentProvider) {
        BillingInfo shippingInfo = createBillingInfo(deliveryAddress);

        PaymentTransactionModel transaction = modelService.create(PaymentTransactionModel.class);
        transaction.setCode(merchantTransactionCode);
        transaction.setPlannedAmount(amount);

        return authorizeInternal(transaction, amount, currency, shippingInfo, null, subscriptionID, cv2,
                paymentProvider);
    }

    protected BillingInfo createBillingInfo(AddressModel address) {
        if (address == null) {
            return null;
        }

        BillingInfo billingInfo = new BillingInfo();
        billingInfo.setCity(address.getTown());

        if (address.getCountry() != null) {
            billingInfo.setCountry(address.getCountry().getIsocode());
        }

        billingInfo.setEmail(address.getEmail());
        billingInfo.setFirstName(address.getFirstname());
        billingInfo.setLastName(address.getLastname());
        billingInfo.setPhoneNumber(address.getPhone1());
        billingInfo.setPostalCode(address.getPostalcode());

        if (address.getRegion() != null) {
            billingInfo.setState(address.getRegion().getName());
        }

        billingInfo.setStreet1(address.getStreetname());
        billingInfo.setStreet2(address.getStreetnumber());

        return billingInfo;
    }

    protected PaymentTransactionEntryModel authorizeInternal(PaymentTransactionModel transaction, BigDecimal amount,
                                                           Currency currency, BillingInfo shippingInfo, CardInfo card,
                                                           String subscriptionID, String cv2, String paymentProvider) {

        CartModel cartModel = cartService.getSessionCart();
        PaymentTransactionType paymentTransactionType = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(cartModel.getPaymentInfo(), cartModel.getSite());
        String newEntryCode = getNewPaymentTransactionEntryCode(transaction, paymentTransactionType);
        AuthorizationResult result;

        if (subscriptionID == null) {
            result = cardPaymentService.authorize(new AuthorizationRequest(newEntryCode, card, currency, amount,
                    shippingInfo));
        } else {
            String orderId = pspUtils.generateUniqueOrderId(cartModel);

            result = cardPaymentService.authorize(new PspSubscriptionAuthorizationRequest(newEntryCode, subscriptionID,
                    currency, amount, shippingInfo, cv2, paymentProvider, orderId, card));
        }

        transaction.setRequestId(result.getRequestId());
        transaction.setRequestToken(result.getRequestToken());
        transaction.setPaymentProvider(paymentProvider);
        modelService.save(transaction);

        PaymentTransactionEntryModel entry = savePaymentTransactionEntry(transaction, subscriptionID, paymentTransactionType, newEntryCode, result);
        modelService.refresh(transaction);

        return entry;
    }


    private PaymentTransactionEntryModel savePaymentTransactionEntry(PaymentTransactionModel transaction, String subscriptionID, PaymentTransactionType paymentTransactionType, String newEntryCode, AuthorizationResult result) {
        PaymentTransactionEntryModel entry = modelService.create(PaymentTransactionEntryModel.class);
        entry.setAmount(result.getTotalAmount());

        if (result.getCurrency() != null) {
            entry.setCurrency(commonI18NService.getCurrency(result.getCurrency().getCurrencyCode()));
        }

        entry.setType(paymentTransactionType);
        entry.setTime((result.getAuthorizationTime() == null) ? new Date() : result.getAuthorizationTime());
        entry.setPaymentTransaction(transaction);
        entry.setRequestId(result.getRequestId());
        entry.setRequestToken(result.getRequestToken());
        entry.setTransactionStatusDetails(result.getTransactionStatusDetails().name());
        entry.setCode(newEntryCode);
        CartModel cartModel = cartService.getSessionCart();
        if (result instanceof PspAuthorizationResult) {
            PspAuthorizationResult pspAuthorizationResult = (PspAuthorizationResult) result;
            cartModel.setPspIntegrationResult(pspAuthorizationResult.getHtmlAnswer());
            cartModel.setPaymentErrorCode(pspAuthorizationResult.getNcError());
            cartModel.setPaymentErrorStatus(pspAuthorizationResult.getNcStatus());
            entry.setTransactionStatus(pspAuthorizationResult.getPspTransactionStatus());
        }

        if (subscriptionID != null) {
            entry.setSubscriptionID(subscriptionID);
        }
        modelService.saveAll(entry, cartModel);
        return entry;
    }

    @Override
    public AbstractFlexPaymentInfoModel createPaymentInfo(Map<String, String> response) {
        validateParameterNotNull(response, "response Map cannot be null");

        CartModel cartModel = cartService.getSessionCart();
        PaymentModeModel paymentModeModel = cartModel.getPaymentMode();
        ComposedTypeModel composedTypeModel = paymentModeModel.getPaymentInfoType();
        PspFlexPaymentInfoCreateStrategy pspFlexPaymentInfoCreateStrategy = (PspFlexPaymentInfoCreateStrategy) pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel);
        AbstractFlexPaymentInfoModel flexPaymentInfoModel = pspFlexPaymentInfoCreateStrategy.saveSubscription(cartModel);
        flexPaymentInfoModel.setPayId(response.get(PspConstants.KEY_PAYID));
        flexPaymentInfoModel.setAcceptanceCode(response.get(PspConstants.KEY_RESULT_ACCEPTANCE));
        flexPaymentInfoModel.setBillingAddress(cartModel.getPaymentAddress());
        flexPaymentInfoModel.setBrand(response.get(PspConstants.KEY_RESULT_BRAND));
        flexPaymentInfoModel.setCardNumber(response.get(PspConstants.KEY_CARDNO));
        cartModel.setPaymentInfo(flexPaymentInfoModel);
        modelService.saveAll(flexPaymentInfoModel, cartModel);

        return flexPaymentInfoModel;
    }

    public void setModelService(ModelService modelService) {
        this.modelService = modelService;
    }

    public void setCardPaymentService(CardPaymentService cardPaymentService) {
        this.cardPaymentService = cardPaymentService;
    }

    public void setCommonI18NService(CommonI18NService commonI18NService) {
        this.commonI18NService = commonI18NService;
    }

    @Override
    public void processPostPaymentFeedback(Map<String, String> parameters) {
        if (pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_DL_SHA_SIGNATURE)
                && parameters.containsKey(PspConstants.KEY_STATUS)) {
            pspPostPaymentFeedbackStrategy.handleFeedback(parameters);
        }
    }
}
