package com.psp.payment.common.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.strategies.PspSignatureValidationStrategy;
import com.psp.payment.common.utils.PspUtils;
import de.hybris.platform.util.Config;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.RuleBasedCollator;
import java.util.Map;
import java.util.TreeMap;

/**
 * Implementation for PSP signature validation against parameters received, based on a shared secret.
 */
public class PspSignatureValidationStrategyImpl implements PspSignatureValidationStrategy {

    private static final Logger LOG = LoggerFactory.getLogger(PspSignatureValidationStrategyImpl.class);

    @Resource
    private PspUtils pspUtils;

    @Override
    public boolean validateSignature(Map<String, String> parameters, String signatureFieldName) {
        boolean validationResult = false;

        if (MapUtils.isNotEmpty(parameters) && StringUtils.isNotEmpty(signatureFieldName)) {
            String passPhrase = getParameter(PspConstants.PARAM_PASSPHRASE);
            String receivedShaSign = parameters.get(signatureFieldName);
            String calculatedShaSign = null;

            if (StringUtils.isNotEmpty(passPhrase) && StringUtils.isNotEmpty(receivedShaSign)) {
                Map<String, String> filteredParameters = filterParameters(parameters, signatureFieldName);

                if (MapUtils.isNotEmpty(filteredParameters)) {
                    calculatedShaSign = pspUtils.generateChecksum(filteredParameters, passPhrase);
                    validationResult = StringUtils.equals(calculatedShaSign, receivedShaSign);
                }
            }

            if (!validationResult) {
                LOG.error("Cannot process PSP response. Calculated SHA is [" + calculatedShaSign + "]. Received SHA is [" + receivedShaSign + "]");
            }
        }

        return validationResult;
    }

    public String getParameter(String key) {
        return Config.getParameter(key);
    }

    /**
     * Filters the parameters not used for signature generation.
     *
     * @param parameters original parameters
     * @return parameters used for signature generation
     */
    private Map<String, String> filterParameters(Map<String, String> parameters, String signatureField) {
        Map<String, String> filteredParameters = null;

        try {
            String rules = "< '_' < a";
            filteredParameters = new TreeMap<>(new RuleBasedCollator(rules));

            for (Map.Entry<String, String> paramterEntry : parameters.entrySet()) {
                if (!paramterEntry.getKey().equalsIgnoreCase(signatureField)
                        && !paramterEntry.getKey().equalsIgnoreCase("mainWindow")
                        && StringUtils.isNotEmpty(paramterEntry.getValue())) {
                    filteredParameters.put(paramterEntry.getKey(), paramterEntry.getValue());
                }
            }
        } catch (ParseException e) {
            LOG.error("Collator rule parsing error. This should never be logged.");
        }

        return filteredParameters;
    }

    public void setPspUtils(PspUtils pspUtils) {
        this.pspUtils = pspUtils;
    }
}
