package com.psp.payment.common.utils;

import com.psp.constants.PspConstants;
import de.hybris.platform.core.model.order.AbstractOrderModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConversionException;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;
import org.springframework.util.Base64Utils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;
import java.util.NoSuchElementException;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/**
 * Utility class for the PSP.
 */
public class PspUtils {

    public static final Logger LOGGER = LoggerFactory.getLogger(PspUtils.class);

    @Resource
    private ModelService modelService;

    @Resource
    private ConfigurationService configurationService;

    /**
     * Generates the checksum for given parameters and the secret key. All the
     * entries with non-null values in parameters will be used to generate the
     * checksum.
     *
     * @param parameterMap the parameters to include in the checksum, TreeMap implementation should be passed on in case sorting needs to be maintained
     * @param secretKey    the secret key
     * @return the generated hexadecimal checksum in uppercase
     */
    public String generateChecksum(Map<String, String> parameterMap, String secretKey) {
        Assert.notNull(parameterMap);
        Assert.notNull(secretKey);

        String checksum = PspConstants.EMPTY_STRING;
        StringBuilder stringToHash = new StringBuilder();

        for (Map.Entry<String, String> paramterEntry : parameterMap.entrySet()) {
            if (StringUtils.isNotEmpty(paramterEntry.getValue())) {
                stringToHash.append(paramterEntry.getKey());
                stringToHash.append(PspConstants.EQUALS);
                stringToHash.append(paramterEntry.getValue());
                stringToHash.append(secretKey);
            }
        }

        String hashString = stringToHash.toString();

        if (StringUtils.isNotEmpty(hashString)) {
            try {
                MessageDigest msgDigest = MessageDigest.getInstance(PspConstants.ALGORITHM_SHA256);
                msgDigest.update(hashString.getBytes());
                byte[] digestBytes = msgDigest.digest();
                StringBuilder hexFormatBuilder = new StringBuilder();

                for (int i = 0; i < digestBytes.length; i++) {
                    hexFormatBuilder.append(String.format(PspConstants.HEX_FORMAT, digestBytes[i]));
                }

                checksum = StringUtils.upperCase(hexFormatBuilder.toString());
            } catch (NoSuchAlgorithmException nsae) {
                LOGGER.error("Cannot generate checksum", nsae);
            }
        }

        return checksum;
    }

    /**
     * Generates the unique order id for the given cart or order. Order id
     * pattern will be code_ddMMyy-HHmmss.
     *
     * @param abstractOrderModel the cart or order
     * @return unique order id
     */
    public String generateUniqueOrderId(AbstractOrderModel abstractOrderModel) {
        validateParameterNotNull(abstractOrderModel, "Parameter [abstractOrderModel] cannot be null.");

        DateFormat currentTimeFormat = new SimpleDateFormat("ddMMyy-HHmmss");
        Date currentDate = Calendar.getInstance().getTime();
        String formattedCurrentTime = currentTimeFormat.format(currentDate);

        return abstractOrderModel.getCode().concat(PspConstants.HYPHEN).concat(formattedCurrentTime);
    }

    public boolean is3DSecureEnabled() {
        boolean isEnabled = false;

        try {
            isEnabled = getConfiguration().getBoolean(PspConstants.PARAM_3D_SECURE_ENABLED);
        } catch (NoSuchElementException | ConversionException exception) {
            LOGGER.error("3D Secure enabled property is not being set. Setting it to disabled.");
        }

        return isEnabled;
    }

    public String decodeUsingBase64Encryption(String encodedText) {
        String decodedText = StringUtils.EMPTY;

        if (StringUtils.isNotEmpty(encodedText)) {
            byte[] decodedBytes = Base64Utils.decodeFromString(encodedText);

            if (decodedBytes != null) {
                decodedText = new String(decodedBytes);
            }
        }

        return decodedText;
    }

    public String getAmountMultipliedBy100(BigDecimal amount) {
        validateParameterNotNull(amount, "Parameter [amount] is not null.");

        if (amount.signum() == -1) {
            throw new IllegalArgumentException("Amount cannot be negative.");
        }

        BigDecimal cartAmountX100 = amount.scaleByPowerOfTen(2);
        return cartAmountX100.toPlainString();
    }

    public String getWebsiteIdByCart(AbstractOrderModel abstractOrderModel) {
        validateParameterNotNull(abstractOrderModel, "Parameter [abstractOrderModel] is not null.");
        String cmsSiteUID = StringUtils.EMPTY;

        if (abstractOrderModel.getSite() != null) {
            cmsSiteUID = abstractOrderModel.getSite().getUid();
        }

        return cmsSiteUID;
    }

    public Configuration getConfiguration() {
        return configurationService.getConfiguration();
    }
}
