package com.psp.payment.directlink.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.dto.Psp3DSecureDTO;
import com.psp.payment.directlink.dto.PspSubscriptionAuthorizationRequest;
import com.psp.payment.directlink.request.PspDirectLinkRequest;
import com.psp.payment.directlink.strategies.PspAddressBuilderStrategy;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.commerceservices.i18n.CommerceCommonI18NService;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.dto.BillingInfo;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.session.SessionService;
import de.hybris.platform.site.BaseSiteService;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.MediaType;

import javax.annotation.Resource;

import static com.psp.constants.PspConstants.*;
import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;
import static java.lang.Boolean.TRUE;

/**
 * Populator for creating the request for directlink rest call.
 */
public class PspDirectLinkRequestPopulator implements Populator<PspSubscriptionAuthorizationRequest, PspDirectLinkRequest> {

    @Resource
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Resource
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Resource
    private PspAddressBuilderStrategy pspAddressBuilderStrategy;

    @Resource
    private PspUtils pspUtils;

    @Resource
    private CommerceCommonI18NService commerceCommonI18NService;

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Resource
    private CartService cartService;

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Resource
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource
    private BaseSiteService baseSiteService;

    @Resource
    private SessionService sessionService;

    @Override
    public void populate(PspSubscriptionAuthorizationRequest source, PspDirectLinkRequest target) {
        validateParameterNotNull(source, "Parameter [PspSubscriptionAuthorizationRequest] source cannot be null");
        validateParameterNotNull(target, "Parameter [DirectLinkPaymentRequest] target cannot be null");

        setStaticParameters(target);
        target.setCurrency(source.getCurrency().getCurrencyCode());
        target.setOrderId(source.getOrderId());

        String cartAmountX100 = pspUtils.getAmountMultipliedBy100(source.getTotalAmount());
        target.setAmount(Integer.parseInt(cartAmountX100));
        target.setAlias(source.getSubscriptionID());
        populateName(target);
        populateAddress(source, target);

        if (pspUtils.is3DSecureEnabled()) {
            populate3DSecureFields(target);
        }
    }

    private void populateName(PspDirectLinkRequest target) {
        PaymentInfoModel paymentInfoModel = cartService.getSessionCart().getPaymentInfo();

        if (paymentInfoModel instanceof CreditCardPaymentInfoModel) {
            String customerName = ((CreditCardPaymentInfoModel) paymentInfoModel).getCcOwner();

            if (StringUtils.isNotEmpty(customerName)) {
                target.setCustomerName(customerName);
            }
        }
    }

    private void populateAddress(PspSubscriptionAuthorizationRequest source, PspDirectLinkRequest target) {
        BillingInfo shippingAddress = source.getShippingInfo();

        if (shippingAddress != null) {
            String email = shippingAddress.getEmail();

            if (StringUtils.isEmpty(email)) {
                email = customerEmailResolutionService.getEmailForCustomer(checkoutCustomerStrategy.getCurrentUserForCheckout());
            }

            target.setEmail(email);
            target.setAddress(pspAddressBuilderStrategy.buildAddress(shippingAddress.getStreet1(), shippingAddress.getStreet2()));
            target.setPostCode(shippingAddress.getPostalCode());
            target.setTown(shippingAddress.getCity());
            target.setCountry(shippingAddress.getCountry());
            target.setPhoneNumber(shippingAddress.getPhoneNumber());
        }
    }

    private void populate3DSecureFields(PspDirectLinkRequest target) {
        BaseSiteModel currentBaseSite = baseSiteService.getCurrentBaseSite();
        Configuration configuration = configurationService.getConfiguration();
        String acceptUrl = siteBaseUrlResolutionService
                .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_3DSECURE_ACCEPT_URL));
        String exceptionUrl = siteBaseUrlResolutionService
                .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_3DSECURE_EXCEPTION_URL));
        String declineUrl = siteBaseUrlResolutionService
                .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_3DSECURE_DECLINE_URL));
        Psp3DSecureDTO psp3DSecureDTO = sessionService.getCurrentSession().getAttribute(PSP_3D_SECURE);

        target.setFlag3d(PspConstants.VALUE_FLAG3D);
        target.setAcceptUrl(acceptUrl);
        target.setExceptionUrl(exceptionUrl);
        target.setHttpAccept(MediaType.ALL_VALUE);
        target.setWin3ds(configuration.getString(PspConstants.PARAM_WIN3DS));
        target.setDeclineUrl(declineUrl);
        target.setLanguage(commerceCommonI18NService.getCurrentLanguage().getIsocode());
        target.setBrowserAcceptHeader(psp3DSecureDTO.getBrowserAcceptHeader());
        target.setBrowserColorDepth(psp3DSecureDTO.getBrowserColorDepth());
        target.setBrowserLanguage(psp3DSecureDTO.getBrowserLanguage());
        target.setBrowserScreenHeight(psp3DSecureDTO.getBrowserScreenHeight());
        target.setBrowserScreenWidth(psp3DSecureDTO.getBrowserScreenWidth());
        target.setBrowserTimeZone(psp3DSecureDTO.getBrowserTimeZone());
        target.setBrowserUserAgent(psp3DSecureDTO.getBrowserUserAgent());
        target.setBrowserJavaEnabled(psp3DSecureDTO.getBrowserJavaEnabled());
        target.setChallengeWindowSize(psp3DSecureDTO.getChallengeWindowSize());
        //target.setRemoteAddr(psp3DSecureDTO.getRemoteAddr());
    }

    private void setStaticParameters(PspDirectLinkRequest target) {
        target.setPspId(configurationService.getConfiguration().getString(PARAM_PSPID));
        target.setUserId(configurationService.getConfiguration().getString(PARAM_API_USER_ID));
        target.setPassword(configurationService.getConfiguration().getString(PARAM_API_USER_PASSWORD));
        target.setEci(PspConstants.VALUE_ECI_RECURRING);
        target.setOperation(pspPaymentProcessService.findOperationForPaymentMode(cartService.getSessionCart().getPaymentMode()).name());
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        if (StringUtils.isNotEmpty(trackingKey)) {
            target.getParameters().put(PspConstants.KEY_TRACKING, trackingKey);
        }
    }
}
