package com.psp.payment.directlink.services.impl;

import com.psp.constants.PspConstants;
import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.exception.RequiredFieldException;
import com.psp.payment.directlink.request.PspDirectLinkRequest;
import com.psp.payment.directlink.response.PspDirectLinkResponse;
import com.psp.payment.directlink.services.PspDirectLinkService;
import com.psp.payment.integration.clients.PspRestClient;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import com.psp.payment.integration.utils.PspIntegrationUtils;
import de.hybris.platform.commerceservices.order.CommerceCheckoutService;
import de.hybris.platform.commerceservices.strategies.GenerateMerchantTransactionCodeStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.log4j.Logger;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.*;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;
import static org.springframework.util.StringUtils.hasText;


/**
 * Service class for communication with the psp directlink server-to-server request.
 */
public class PspDirectLinkServiceImpl implements PspDirectLinkService {

    private static final Logger LOGGER = Logger.getLogger(PspDirectLinkServiceImpl.class);

    @Resource
    private PspUtils pspUtils;

    @Resource
    private CartService cartService;

    @Resource
    private ModelService modelService;

    @Resource
    private CommonI18NService commonI18NService;

    @Resource
    private PaymentService paymentService;

    @Resource
    private GenerateMerchantTransactionCodeStrategy generateMerchantTransactionCodeStrategy;

    @Resource
    private CommerceCheckoutService commerceCheckoutService;

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private PspRestClient<String> pspRestClient;

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Resource
    private PspIntegrationUtils pspIntegrationUtils;

    @Override
    public PspDirectLinkResponse executeDirectLinkRequest(PspDirectLinkRequest pspDirectLinkRequest) {
        validateParameterNotNull(pspDirectLinkRequest, "Parameter [PspDirectLinkRequest] cannot be null");

        PspDirectLinkResponse pspDirectLinkResponse = new PspDirectLinkResponse();
        Map<String, String> request = toUpperCaseKeys(pspDirectLinkRequest.getParameters());
        String shaSign = pspUtils.generateChecksum(request, getParameter(PspConstants.PARAM_PASSPHRASE));
        request.put(PspConstants.KEY_DL_SHA_SIGNATURE, shaSign);

        String response = pspRestClient.postRequest(getUri(), request, String.class);
        Map<String, String> responseParameters = pspIntegrationUtils.convertPlainStringToParameters(response);
        pspDirectLinkResponse.setParameters(responseParameters);

        return pspDirectLinkResponse;
    }

    @Override
    public PaymentTransactionEntryModel createPaymentTransactionAndEntry(Map<String, String> parameters) {
        CartModel cartModel = cartService.getSessionCart();
        PaymentTransactionType paymentTransactionType = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(cartModel.getPaymentInfo(), cartModel.getSite());

        PaymentTransactionModel paymentTransactionModel = modelService.create(PaymentTransactionModel.class);
        paymentTransactionModel.setCode(generateMerchantTransactionCodeStrategy.generateCode(cartModel));
        paymentTransactionModel.setCurrency(commonI18NService.getCurrency(parameters.get(PspConstants.KEY_CURRENCY)));
        paymentTransactionModel.setRequestId(parameters.get(PspConstants.KEY_PAYID));
        paymentTransactionModel.setPaymentProvider(commerceCheckoutService.getPaymentProvider());
        paymentTransactionModel.setRequestToken(parameters.get(PspConstants.KEY_PAYID));
        paymentTransactionModel.setOrder(cartModel);
        paymentTransactionModel.setInfo(cartModel.getPaymentInfo());

        PaymentTransactionEntryModel paymentTransactionEntryModel = modelService.create(PaymentTransactionEntryModel.class);
        paymentTransactionEntryModel.setCode(paymentService.getNewPaymentTransactionEntryCode(paymentTransactionModel, paymentTransactionType));
        paymentTransactionEntryModel.setType(paymentTransactionType);
        paymentTransactionEntryModel.setRequestId(parameters.get(PspConstants.KEY_PAYID));
        paymentTransactionEntryModel.setRequestToken(parameters.get(PspConstants.KEY_PAYID));
        paymentTransactionEntryModel.setTime(new Date());
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatusDetails.SUCCESFULL.name());
        paymentTransactionEntryModel.setCurrency(commonI18NService.getCurrency(parameters.get(PspConstants.KEY_CURRENCY)));

        paymentTransactionEntryModel.setTransactionStatus(getTransactionStatus(parameters));

        String amount = MapUtils.getString(parameters, PspConstants.KEY_RESULT_AMOUNT);
        paymentTransactionEntryModel.setAmount(getAmount(amount));
        paymentTransactionModel.setPlannedAmount(getAmount(amount));

        modelService.saveAll(paymentTransactionEntryModel, paymentTransactionModel);

        return paymentTransactionEntryModel;
    }

    @Override
    public String getTransactionStatus(Map<String, String> parameters) {
        String transactionStatus;
        String status = MapUtils.getString(parameters, PspConstants.KEY_STATUS);
        String fraudScore = MapUtils.getString(parameters, PspConstants.KEY_RESULT_FRAUD_CATEGORY);
        int statusAsInt = Integer.valueOf(status);
        boolean isFraudCheckEnabled = StringUtils.isEmpty(fraudScore) ? Boolean.FALSE : Boolean.TRUE;

        switch (statusAsInt) {
            case PspPaymentStatus.STATUS_AUTHORISED:
            case PspPaymentStatus.STATUS_PAYMENT_REQUESTED: {
                transactionStatus = getTransactionStatusBasedOnFraudScore(fraudScore, isFraudCheckEnabled).name();
                break;
            }
            case PspPaymentStatus.STATUS_IDENTIFICATION_REQUESTED: {
                transactionStatus = TransactionStatus.ACCEPTED.name();
                break;
            }
            case PspPaymentStatus.STATUS_AUTHORISATION_NOT_KNOWN:
            case PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN:
            case PspPaymentStatus.STATUS_AUTHORISATION_WAITING:
            case PspPaymentStatus.STATUS_PAYMENT_PROCESSING: {
                if (configurationService.getConfiguration().getBoolean(PspConstants.PARAM_UNCERTAIN_STATUS_HANDLING_ENABELD)) {
                    transactionStatus = PspTransactionStatus.UNCERTAIN.name();
                    break;
                } else {
                    transactionStatus = getTransactionStatusBasedOnFraudScore(fraudScore, isFraudCheckEnabled).name();
                    break;
                }
            }
            default: {
                transactionStatus = TransactionStatus.REJECTED.name();
            }
        }

        return transactionStatus;
    }

    @Override
    public TransactionStatusDetails getTransactionStatusDetails(Map<String, String> parameters) {
        TransactionStatusDetails transactionStatusDetails;
        String fraudScore = MapUtils.getString(parameters, PspConstants.KEY_RESULT_FRAUD_CATEGORY);
        boolean isFraudCheckEnabled = StringUtils.isEmpty(fraudScore) ? Boolean.FALSE : Boolean.TRUE;
        String status = MapUtils.getString(parameters, PspConstants.KEY_STATUS);
        int statusAsInt = Integer.valueOf(status);
        switch (statusAsInt) {
            case PspPaymentStatus.STATUS_IDENTIFICATION_REQUESTED: {
                transactionStatusDetails = TransactionStatusDetails.THREE_D_SECURE_AUTHENTICATION_REQUIRED;
                break;
            }
            case PspPaymentStatus.STATUS_AUTHORISED:
            case PspPaymentStatus.STATUS_PAYMENT_REQUESTED: {
                transactionStatusDetails = TransactionStatusDetails.SUCCESFULL;
                break;
            }
            case PspPaymentStatus.STATUS_AUTHORISATION_NOT_KNOWN:
            case PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN:
            case PspPaymentStatus.STATUS_AUTHORISATION_WAITING:
            case PspPaymentStatus.STATUS_PAYMENT_PROCESSING: {
                transactionStatusDetails = getTransactionStatusDetailsBasedOnFraudScore(fraudScore, isFraudCheckEnabled);
                break;
            }
            default: {
                transactionStatusDetails = TransactionStatusDetails.AUTHORIZATION_REJECTED_BY_PSP;
                break;
            }
        }
        return transactionStatusDetails;
    }

    private TransactionStatusDetails getTransactionStatusDetailsBasedOnFraudScore(String fraudScore, boolean isFraudCheckEnabled) {
        TransactionStatusDetails transactionStatusDetails;
        if (isFraudCheckEnabled) {
            switch (fraudScore) {
                case PspConstants.VALUE_SCO_SCORE_GREEN: {
                    transactionStatusDetails = TransactionStatusDetails.SUCCESFULL;
                    break;
                }
                case PspConstants.VALUE_SCO_SCORE_ORANGE: {
                    transactionStatusDetails = TransactionStatusDetails.REVIEW_NEEDED;
                    break;
                }
                default: {
                    transactionStatusDetails = TransactionStatusDetails.AUTHORIZATION_REJECTED_BY_PSP;
                }
            }
        } else {
            transactionStatusDetails = TransactionStatusDetails.SUCCESFULL;
        }
        return transactionStatusDetails;
    }

    private BigDecimal getAmount(String amount) {
        BigDecimal transactionAmount = new BigDecimal("0.0");

        if (NumberUtils.isNumber(amount)) {
            transactionAmount = new BigDecimal(amount);
        }

        return transactionAmount;
    }

    private Map<String, String> toUpperCaseKeys(Map<String, String> parameters) {
        validate(parameters);
        Map<String, String> result = new TreeMap<>();

        for (Map.Entry<String, String> entry : parameters.entrySet()) {
            String value = entry.getValue();

            if (hasText(value)) {
                result.put(StringUtils.upperCase(entry.getKey()), value);
            }
        }

        return result;
    }

    private List<String> getMandatoryRequestFields() {
        List<String> fields = new ArrayList<>(Arrays.asList(PspConstants.KEY_PSPID,
                PspConstants.KEY_API_USERID,
                PspConstants.KEY_API_PASSWORD,
                PspConstants.KEY_DL_ORDERID,
                PspConstants.KEY_CURRENCY,
                PspConstants.KEY_AMOUNT,
                PspConstants.KEY_OPERATION,
                PspConstants.KEY_ECI,
                PspConstants.KEY_ALIAS));

        if (pspUtils.is3DSecureEnabled()) {
            fields.addAll(Arrays.asList(PspConstants.KEY_FLAG3D,
                    PspConstants.KEY_HTTP_ACCEPT,
                    PspConstants.KEY_WIN3DS,
                    PspConstants.KEY_ACCEPTURL,
                    PspConstants.KEY_DECLINEURL,
                    PspConstants.KEY_EXCEPTIONURL,
                    PspConstants.KEY_LANGUAGE));
        }

        return fields;
    }

    private boolean validate(Map<String, String> parameters) {
        for (String field : getMandatoryRequestFields()) {
            if (!parameters.containsKey(field)) {
                throw new RequiredFieldException("Field " + field + " value cannot be empty");
            }
        }

        return true;
    }

    private TransactionStatus getTransactionStatusBasedOnFraudScore(String fraudScore, boolean isFraudCheckEnabled) {
        TransactionStatus transactionStatus;
        if (isFraudCheckEnabled) {
            switch (fraudScore) {
                case PspConstants.VALUE_SCO_SCORE_GREEN: {
                    transactionStatus = TransactionStatus.ACCEPTED;
                    break;
                }
                case PspConstants.VALUE_SCO_SCORE_ORANGE: {
                    transactionStatus = TransactionStatus.REVIEW;
                    break;
                }
                default: {
                    transactionStatus = TransactionStatus.REJECTED;
                }
            }
        } else {
            transactionStatus = TransactionStatus.ACCEPTED;
        }
        return transactionStatus;
    }

    private URI getUri() {
        URI uri = null;
        String url = getParameter(PspConstants.PARAM_DIRECTLINK_REQUEST_URL);

        if (StringUtils.isNotEmpty(url)) {
            try {
                uri = new URI(url);
            } catch (URISyntaxException exception) {
                LOGGER.error("Invalid URL found for DirectLink request.", exception);
            }
        }

        return uri;
    }

    private String getParameter(String key) {
        return configurationService.getConfiguration().getString(key);
    }
}
