package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.constants.PspConstants;
import com.psp.payment.fraud.PspFraudService;
import de.hybris.platform.basecommerce.enums.FraudStatus;
import de.hybris.platform.core.enums.OrderStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.fraud.model.FraudReportModel;
import de.hybris.platform.orderhistory.model.OrderHistoryEntryModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.processengine.action.AbstractAction;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.util.ServicesUtil;
import de.hybris.platform.task.RetryLaterException;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.Set;

public class PspFraudCheckOrderAction extends AbstractAction<OrderProcessModel> {

    @Resource
    private PspFraudService fraudService;

    @Resource
    private ConfigurationService configurationService;

    private String providerName;

    @Override
    public final String execute(final OrderProcessModel process) throws RetryLaterException, Exception {
        return executeAction(process).toString();
    }

    protected Transition executeAction(OrderProcessModel process) throws RetryLaterException, Exception {
        ServicesUtil.validateParameterNotNull(process, "Process can not be null");
        ServicesUtil.validateParameterNotNull(process.getOrder(), "Order can not be null");

        final double scoreLimit = configurationService.getConfiguration().getDouble(PspConstants.PARAM_FRAUD_SCORE_LIMIT);
        final double scoreTolerance = configurationService.getConfiguration().getDouble(PspConstants.PARAM_FRAUD_SCORE_TOLERANCE);

        final OrderModel order = process.getOrder();
        final FraudServiceResponse response = fraudService.recognizeOrderSymptoms(getProviderName(), order);

        final double score = response.getScore();
        if (score < scoreLimit) {
            processFraudScore(order, response, FraudStatus.OK, Boolean.FALSE, Boolean.FALSE);
            return Transition.OK;
        } else if (score < scoreLimit + scoreTolerance) {
            processFraudScore(order, response, FraudStatus.CHECK, Boolean.FALSE, Boolean.TRUE);
            return Transition.POTENTIAL;
        } else {
            processFraudScore(order, response, FraudStatus.FRAUD, Boolean.TRUE, Boolean.FALSE);
            return Transition.FRAUD;
        }
    }

    private void processFraudScore(OrderModel order, FraudServiceResponse response, FraudStatus fraudStatus, Boolean fraudulent, Boolean potentiallyFraudulent) {
        FraudReportModel fraudReport = fraudService.createFraudReport(getProviderName(), response, order, fraudStatus);
        OrderHistoryEntryModel historyEntry = fraudService.createHistoryLog(getProviderName(), order, fraudStatus,
                fraudReport.getCode());
        order.setFraudulent(fraudulent);
        order.setPotentiallyFraudulent(potentiallyFraudulent);
        order.setStatus(OrderStatus.FRAUD_CHECKED);
        modelService.saveAll(fraudReport, historyEntry, order);
    }

    public enum Transition {
        OK, POTENTIAL, FRAUD;

        public static Set<String> getStringValues() {
            final Set<String> res = new HashSet<String>();
            for (final Transition transitions : Transition.values()) {
                res.add(transitions.toString());
            }
            return res;
        }
    }

    @Override
    public Set<String> getTransitions() {
        return Transition.getStringValues();
    }

    protected String getProviderName() {
        return providerName;
    }

    public void setProviderName(final String providerName) {
        this.providerName = providerName;
    }
}
