package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspOperationEvaluationStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.BooleanUtils;
import org.apache.log4j.Logger;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static com.psp.payment.common.enums.PspOperation.SAL;
import static com.psp.payment.common.enums.PspOperation.SAS;
import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspCaptureEvaluationStrategyImpl implements PspOperationEvaluationStrategy {

    private static final Logger LOGGER = Logger.getLogger(PspCaptureEvaluationStrategyImpl.class);

    @Override
    public PspOperation evaluateOperation(OrderModel orderModel, BigDecimal proposedCaptureAmount) {
        validateParameters(orderModel, proposedCaptureAmount);

        List<PaymentTransactionEntryModel> authorisePaymentTransactionEntries = new ArrayList<>();
        List<PaymentTransactionEntryModel> capturePaymentTransactionEntries = new ArrayList<>();
        List<PaymentTransactionModel> paymentTransactionModelList = orderModel.getPaymentTransactions();
        populatePaymentTransactions(paymentTransactionModelList, authorisePaymentTransactionEntries, capturePaymentTransactionEntries);

        if (CollectionUtils.isEmpty(authorisePaymentTransactionEntries)) {
            throw new IllegalMaintenanceOperation("No authorise transaction entries found for the order.");
        }

        BigDecimal totalAuthoriseAmount = calculateTotalsForPaymentTransactionType(authorisePaymentTransactionEntries);

        if (totalAuthoriseAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalMaintenanceOperation("Insufficient authorisation amount found for the order.");
        }

        BigDecimal totalCaptureAmount = calculateTotalsForPaymentTransactionType(capturePaymentTransactionEntries);
        BigDecimal remainingCaptureAmount = totalAuthoriseAmount.subtract(totalCaptureAmount);

        if (remainingCaptureAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalMaintenanceOperation("Full amount has already been captured.");
        }

        if (remainingCaptureAmount.compareTo(proposedCaptureAmount) < 0) {
            throw new IllegalMaintenanceOperation("Insufficient amount left to capture.");
        }

        PspOperation pspOperation = getPspOperation(proposedCaptureAmount, remainingCaptureAmount);
        Boolean partialMaintenanceAllowed = orderModel.getPaymentMode().getPartialMaintenanceAllowed();

        if (SAL.compareTo(pspOperation) == 0 && BooleanUtils.isNotTrue(partialMaintenanceAllowed)) {
            throw new IllegalMaintenanceOperation("Partial capture is not allowed for the chosen payment mode.");
        }

        return pspOperation;
    }

    private void validateParameters(OrderModel orderModel, BigDecimal amount) {
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null");
        validateParameterNotNull(amount, "Parameter [amount] cannot be null");

        if (amount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalMaintenanceOperation("Amount should be greater than zero.");
        }

        if (CollectionUtils.isEmpty(orderModel.getPaymentTransactions())) {
            throw new IllegalMaintenanceOperation("No payment transactions found for the order.");
        }
    }

    private void populatePaymentTransactions(List<PaymentTransactionModel> paymentTransactionModelList,
                                             List<PaymentTransactionEntryModel> authorisePaymentTransactionEntries,
                                             List<PaymentTransactionEntryModel> capturePaymentTransactionEntries) {
        for (PaymentTransactionModel paymentTransactionModel : paymentTransactionModelList) {
            List<PaymentTransactionEntryModel> entries = paymentTransactionModel.getEntries();

            for (PaymentTransactionEntryModel entry : entries) {
                PaymentTransactionType entryType = entry.getType();

                switch (entryType) {
                    case CAPTURE: {
                        capturePaymentTransactionEntries.add(entry);
                        break;
                    }
                    case AUTHORIZATION: {
                        authorisePaymentTransactionEntries.add(entry);
                        break;
                    }
                    default: {
                        LOGGER.info("Ignoring payment transaction entry as it is not authorise or capture.");
                    }
                }
            }
        }
    }

    private BigDecimal calculateTotalsForPaymentTransactionType(List<PaymentTransactionEntryModel> paymentTransactionEntryModelList) {
        BigDecimal totalAuthoriseAmount = BigDecimal.ZERO;

        for (PaymentTransactionEntryModel paymentTransactionEntryModel : paymentTransactionEntryModelList) {
            BigDecimal entryAmount = paymentTransactionEntryModel.getAmount();

            if (entryAmount != null) {
                totalAuthoriseAmount = totalAuthoriseAmount.add(entryAmount);
            }
        }

        return totalAuthoriseAmount;
    }

    private PspOperation getPspOperation(BigDecimal proposedCaptureAmount, BigDecimal remainingCaptureAmount) {
        PspOperation pspOperation;

        if (remainingCaptureAmount.compareTo(proposedCaptureAmount) == 0) {
            pspOperation = SAS;
        } else {
            pspOperation = SAL;
        }

        return pspOperation;
    }
}
