package com.psp.payment.maintenance.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspMaintenanceRequestCreationStrategy;
import com.psp.payment.maintenance.strategies.PspOperationEvaluationStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspMaintenanceRequestCreationStrategyImpl implements PspMaintenanceRequestCreationStrategy {

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private PspUtils pspUtils;

    @Resource
    private PspOperationEvaluationStrategy pspCaptureEvaluationStrategy;

    @Resource
    private PspOperationEvaluationStrategy pspRefundEvaluationStrategy;

    @Override
    public PspMaintenanceRequest createMaintenanceRequest(PspMaintenanceRequestType pspMaintenanceRequestType,
                                                          OrderModel orderModel, BigDecimal amount) {
        validateParameterNotNull(pspMaintenanceRequestType, "Parameter [pspMaintenanceRequestType] cannot be null.");
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null.");
        validateParameterNotNull(amount, "Parameter [amount] cannot be null.");

        PspMaintenanceRequest pspMaintenanceRequest = new PspMaintenanceRequest();
        setStaticParameters(pspMaintenanceRequest);
        pspMaintenanceRequest.setAmount(pspUtils.getAmountMultipliedBy100(amount));

        if (PspMaintenanceRequestType.CAPTURE.equals(pspMaintenanceRequestType)) {
            pspMaintenanceRequest.setOperation(pspCaptureEvaluationStrategy.evaluateOperation(orderModel, amount));
        } else {
            pspMaintenanceRequest.setOperation(pspRefundEvaluationStrategy.evaluateOperation(orderModel, amount));
        }

        setPayId(pspMaintenanceRequestType, pspMaintenanceRequest, orderModel);

        return pspMaintenanceRequest;
    }

    private void setPayId(PspMaintenanceRequestType pspMaintenanceRequestType,
                          PspMaintenanceRequest pspMaintenanceRequest, OrderModel orderModel) {
        List<PaymentTransactionModel> paymentTransactionModelList = orderModel.getPaymentTransactions();
        List<PaymentTransactionEntryModel> desiredEntries = new ArrayList<>();

        if (CollectionUtils.isNotEmpty(paymentTransactionModelList)) {
            for (PaymentTransactionModel paymentTransactionModel : paymentTransactionModelList) {
                List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = paymentTransactionModel.getEntries();

                if (CollectionUtils.isNotEmpty(paymentTransactionEntryModelList)) {
                    for (PaymentTransactionEntryModel paymentTransactionEntryModel : paymentTransactionEntryModelList) {
                        PaymentTransactionType entryType = paymentTransactionEntryModel.getType();

                        if ((PspMaintenanceRequestType.CAPTURE.equals(pspMaintenanceRequestType)
                                && PaymentTransactionType.AUTHORIZATION.equals(entryType))
                                || (PspMaintenanceRequestType.REFUND.equals(pspMaintenanceRequestType)
                                && PaymentTransactionType.CAPTURE.equals(entryType))) {
                            desiredEntries.add(paymentTransactionEntryModel);
                        }
                    }
                }
            }
        }

        if (CollectionUtils.isNotEmpty(desiredEntries)
                && ((PspMaintenanceRequestType.CAPTURE.equals(pspMaintenanceRequestType) && desiredEntries.size() == 1)
                || PspMaintenanceRequestType.REFUND.equals(pspMaintenanceRequestType))) {
            PaymentTransactionEntryModel entry = desiredEntries.iterator().next();
            String payId = entry.getRequestToken();

            if (StringUtils.isNotEmpty(payId)) {
                pspMaintenanceRequest.setPayId(payId);
            } else {
                throw new IllegalMaintenanceOperation("Transaction does not have a request token.");
            }
        } else {
            throw new IllegalMaintenanceOperation("Either the order does not have any or has more than one desired transactions.");
        }
    }

    private void setStaticParameters(PspMaintenanceRequest pspMaintenanceRequest) {
        pspMaintenanceRequest.setPspId(getParameter(PspConstants.PARAM_PSPID));
        pspMaintenanceRequest.setUserId(getParameter(PspConstants.PARAM_API_USER_ID));
        pspMaintenanceRequest.setPassword(getParameter(PspConstants.PARAM_API_USER_PASSWORD));
    }

    private String getParameter(String key) {
        return configurationService.getConfiguration().getString(key);
    }
}
