package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspOperationEvaluationStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.BooleanUtils;
import org.apache.log4j.Logger;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static com.psp.payment.common.enums.PspOperation.RFD;
import static com.psp.payment.common.enums.PspOperation.RFS;
import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspRefundEvaluationStrategyImpl implements PspOperationEvaluationStrategy {

    private static final Logger LOGGER = Logger.getLogger(PspRefundEvaluationStrategyImpl.class);

    @Override
    public PspOperation evaluateOperation(OrderModel orderModel, BigDecimal proposedRefundAmount) {
        validateParameters(orderModel, proposedRefundAmount);

        List<PaymentTransactionEntryModel> capturePaymentTransactionEntries = new ArrayList<>();
        List<PaymentTransactionEntryModel> refundPaymentTransactionEntries = new ArrayList<>();
        List<PaymentTransactionModel> paymentTransactionModelList = orderModel.getPaymentTransactions();
        populatePaymentTransactions(paymentTransactionModelList, capturePaymentTransactionEntries,
                refundPaymentTransactionEntries);

        if (CollectionUtils.isEmpty(capturePaymentTransactionEntries)) {
            throw new IllegalMaintenanceOperation("No capture transaction entries found for the order.");
        }

        BigDecimal totalCaptureAmount = calculateTotalsForPaymentTransactionType(capturePaymentTransactionEntries);
        BigDecimal totalRefundAmount = calculateTotalsForPaymentTransactionType(refundPaymentTransactionEntries);
        BigDecimal remainingRefundAmount = totalCaptureAmount.subtract(totalRefundAmount);

        if (remainingRefundAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalMaintenanceOperation("Full amount has already been refunded.");
        }

        if (remainingRefundAmount.compareTo(proposedRefundAmount) < 0) {
            throw new IllegalMaintenanceOperation("Insufficient amount left to refund.");
        }

        PspOperation pspOperation = getPspOperation(proposedRefundAmount, remainingRefundAmount);
        Boolean partialMaintenanceAllowed = orderModel.getPaymentMode().getPartialMaintenanceAllowed();

        if (RFD.compareTo(pspOperation) == 0 && BooleanUtils.isNotTrue(partialMaintenanceAllowed)) {
            throw new IllegalMaintenanceOperation("Partial refund is not allowed for the chosen payment mode.");
        }

        return pspOperation;
    }

    private void validateParameters(OrderModel orderModel, BigDecimal amount) {
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null");
        validateParameterNotNull(amount, "Parameter [amount] cannot be null");

        if (amount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalMaintenanceOperation("Amount should be greater than zero.");
        }

        if (CollectionUtils.isEmpty(orderModel.getPaymentTransactions())) {
            throw new IllegalMaintenanceOperation("No payment transactions found for the order.");
        }
    }

    private void populatePaymentTransactions(List<PaymentTransactionModel> paymentTransactionModelList,
                                             List<PaymentTransactionEntryModel> capturePaymentTransactionEntries,
                                             List<PaymentTransactionEntryModel> refundPaymentTransactionEntries) {
        for (PaymentTransactionModel paymentTransactionModel : paymentTransactionModelList) {
            List<PaymentTransactionEntryModel> entries = paymentTransactionModel.getEntries();

            for (PaymentTransactionEntryModel entry : entries) {
                PaymentTransactionType entryType = entry.getType();

                switch (entryType) {
                    case CAPTURE: {
                        capturePaymentTransactionEntries.add(entry);
                        break;
                    }
                    case REFUND_FOLLOW_ON: {
                        refundPaymentTransactionEntries.add(entry);
                        break;
                    }
                    default: {
                        LOGGER.info("Ignoring payment transaction entry for the type: " + entryType);
                    }
                }
            }
        }
    }

    private BigDecimal calculateTotalsForPaymentTransactionType(List<PaymentTransactionEntryModel> paymentTransactionEntryModelList) {
        BigDecimal totalAmount = BigDecimal.ZERO;

        for (PaymentTransactionEntryModel paymentTransactionEntryModel : paymentTransactionEntryModelList) {
            BigDecimal entryAmount = paymentTransactionEntryModel.getAmount();

            if (entryAmount != null) {
                totalAmount = totalAmount.add(entryAmount);
            }
        }

        return totalAmount;
    }

    private PspOperation getPspOperation(BigDecimal proposedCaptureAmount, BigDecimal remainingCaptureAmount) {
        PspOperation pspOperation;

        if (remainingCaptureAmount.compareTo(proposedCaptureAmount) == 0) {
            pspOperation = RFS;
        } else {
            pspOperation = RFD;
        }

        return pspOperation;
    }
}
