package com.psp.payment.common.facades.order.impl;

import com.psp.data.PspCardAuthorizationData;
import com.psp.enums.PspTransactionStatus;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.dto.PspBrand;
import com.psp.payment.common.dto.PspPaymentMode;
import com.psp.payment.common.dto.PspPaymentModeData;
import com.psp.payment.common.enums.PspAuthResult;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.services.PspCustomerAccountService;
import com.psp.payment.common.services.PspPaymentModeService;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import com.psp.payment.flexpayments.strategies.impl.PspCreateFlexPaymentRequestStrategyImpl;
import com.psp.payment.flexpayments.strategies.impl.PspFlexPaymentInfoConvertStrategyImpl;
import com.psp.payment.flexpayments.strategies.impl.PspPayPalPaymentInfoCreateStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspCreditCardPaymentInfoConvertStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspDirectDebitPaymentInfoConvertStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.commercefacades.order.CartFacade;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.order.CommerceCheckoutService;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCheckoutFacadeImplTest {

    private static final String SUBSCRIPTION_ID = "subscriptionId";
    private static final String PSP_ID = "pspId";
    private static final String DEFAULT_PAYMENT_MODE = "creditcard";

    @InjectMocks
    private PspCheckoutFacadeImpl pspCheckoutFacade;

    @Mock
    private PspUtils pspUtils;

    @Mock
    private CreditCardPaymentInfoModel paymentInfoModel;

    @Mock
    private AddressModel newBillingAddress;

    @Mock
    private PspCreateFlexPaymentRequestStrategyImpl pspCreateFlexPaymentRequestStrategy;

    @Mock
    private Converter<PspFlexPaymentRequest, PaymentData> pspFlexPaymentDataConverter;

    @Mock
    private PspPaymentModeService paymentModeService;

    @Mock
    private Converter<PaymentModeModel, PspPaymentModeData> paymentModeConverter;

    @Mock
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Mock
    private PspFlexPaymentInfoConvertStrategyImpl pspPaypalPaymentInfoConvertStrategy;

    @Mock
    private PspCreditCardPaymentInfoConvertStrategyImpl pspCreditCardPaymentInfoConvertStrategy;

    @Mock
    private PspDirectDebitPaymentInfoConvertStrategyImpl pspDirectDebitPaymentInfoConvertStrategy;

    @Mock
    private ModelService modelService;

    @Mock
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Mock
    private PspPayPalPaymentInfoCreateStrategyImpl pspPayPalPaymentInfoCreateStrategy;

    @Mock
    private ComposedTypeModel composedTypeModel;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private PspCustomerAccountService customerAccountService;

    @Mock
    private CartService cartService;

    @Mock
    private CommerceCheckoutService commerceCheckoutService;

    private CartModel cartModel;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private String securityCode;

    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    private CustomerModel userModel;
    @Mock
    private CartFacade cartFacade;

    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;

    @Before
    public void setUp() {
        securityCode = "123";
        directDebitPaymentInfoModel = new DirectDebitPaymentInfoModel();
        directDebitPaymentInfoModel.setSubscriptionId(SUBSCRIPTION_ID);
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setTransactionStatus(TransactionStatus.ACCEPTED.name());
        paymentData = new PaymentData();
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        pspFlexPaymentRequest.setPspId(PSP_ID);

        populateCartWithPaymentInfo();
        populateUser();
        setupMockingBehaviour();
    }

    @Test
    public void authorizeCardPaymentFor3DSecure() {
        // Given
        String encodedText = "!G^Js234";
        String decodedText = "Hello World";
        cartModel.setPspIntegrationResult(encodedText);
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatusDetails.THREE_D_SECURE_AUTHENTICATION_REQUIRED.name());
        when(pspUtils.decodeUsingBase64Encryption(encodedText)).thenReturn(decodedText);

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.THREEDSECURE, result.getResult());
        assertEquals(decodedText, result.getHtml());
        assertEquals(StringUtils.EMPTY, cartModel.getPspIntegrationResult());
    }

    @Test
    public void authorizeCardPaymentForSuccess() {
        // Given
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatus.ACCEPTED.name());

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.SUCCESS, result.getResult());
    }

    @Test
    public void authorizeCardPaymentDirectDebitsForSuccess() {
        // Given
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatus.ACCEPTED.name());
        cartModel.setPaymentInfo(directDebitPaymentInfoModel);
        when(commerceCheckoutService.authorizePayment(any(CommerceCheckoutParameter.class))).thenReturn(paymentTransactionEntryModel);

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(StringUtils.EMPTY);

        // Then
        assertEquals(PspAuthResult.SUCCESS, result.getResult());
    }

    @Test
    public void authorizeCardPaymentForReview() {
        // Given
        paymentTransactionEntryModel.setTransactionStatus(TransactionStatus.REVIEW.name());
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatusDetails.CARD_USED_TOO_RECENTLY.name());

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.SUCCESS, result.getResult());
    }

    @Test
    public void authorizeCardPaymentForFailure() {
        // Given
        paymentTransactionEntryModel.setTransactionStatus(TransactionStatus.REJECTED.name());
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatusDetails.CREDIT_LIMIT_REACHED.name());

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.FAILURE, result.getResult());
    }

    @Test
    public void billingAddressSavedToPaymentInfo() {
        // Given
        AddressModel billingAddress = new AddressModel();
        cartModel.setPaymentAddress(billingAddress);
        when(modelService.clone(billingAddress)).thenReturn(newBillingAddress);

        // When
        pspCheckoutFacade.saveBillingAddressToPaymentInfo();

        // Then
        verify(modelService).save(newBillingAddress);
        verify(modelService).save(paymentInfoModel);
        verify(newBillingAddress).setOwner(paymentInfoModel);
    }

    @Test
    public void billingAddressNotSavedToPaymentInfoWhenNotPresent() {
        // Given cart without a billing address

        // When
        pspCheckoutFacade.saveBillingAddressToPaymentInfo();

        // Then
        verify(modelService, never()).save(newBillingAddress);
        verify(modelService, never()).save(paymentInfoModel);
        verify(newBillingAddress, never()).setOwner(paymentInfoModel);
    }

    @Test
    public void billingAddressNotSavedToPaymentInfoWithExistingPresent() {
        // Given cart
        AddressModel billingAddress = new AddressModel();
        cartModel.setPaymentAddress(billingAddress);
        paymentInfoModel.setBillingAddress(billingAddress);

        // When
        pspCheckoutFacade.saveBillingAddressToPaymentInfo();

        // Then
        verify(modelService, never()).save(newBillingAddress);
        verify(modelService, never()).save(paymentInfoModel);
        verify(newBillingAddress, never()).setOwner(paymentInfoModel);
    }

    @Test
    public void createFlexPaymentData() {
        // When
        pspCheckoutFacade.createFlexPaymentData();

        // Then
        verify(pspCreateFlexPaymentRequestStrategy).createFlexPaymentRequest(cartModel);
    }

    @Test
    public void createFlexPaymentDataWithNullRequest() {
        // Given
        when(pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel)).thenReturn(null);

        // When
        paymentData = pspCheckoutFacade.createFlexPaymentData();

        // Then
        assertNull(paymentData);
    }

    @Test
    public void createFlexPaymentDataWithValidRequest() {
        // Given
        when(pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel)).thenReturn(pspFlexPaymentRequest);

        // When
        paymentData = pspCheckoutFacade.createFlexPaymentData();

        // Then
        verify(pspFlexPaymentDataConverter).convert(pspFlexPaymentRequest);
    }

    @Test
    public void getPaymentDetailsWithNoPaymentInfo() {
        //Given
        cartModel.setPaymentInfo(null);

        //When
        CCPaymentInfoData ccPaymentInfoData = pspCheckoutFacade.getPaymentDetails();

        // Then
        assertNull(ccPaymentInfoData);
    }

    @Test
    public void getPaymentDetailsWithCCPaymentInfo() {
        // Given
        cartModel.setPaymentInfo(paymentInfoModel);
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(any(PaymentInfoModel.class))).thenReturn(pspCreditCardPaymentInfoConvertStrategy);

        // When
        pspCheckoutFacade.getPaymentDetails();

        // Then
        verify(pspCreditCardPaymentInfoConvertStrategy).convert(paymentInfoModel);
    }

    @Test
    public void getPaymentDetailsWithPayPalPaymentInfo() {
        //Given
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(any(PaymentInfoModel.class))).thenReturn(pspPaypalPaymentInfoConvertStrategy);

        // Given
        cartModel.setPaymentInfo(payPalPaymentInfoModel);

        // When
        pspCheckoutFacade.getPaymentDetails();

        // Then
        verify(pspPaypalPaymentInfoConvertStrategy).convert(payPalPaymentInfoModel);
    }

    @Test
    public void getPaymentDetailsWithDirectDebits() {
        // Given
        cartModel.setPaymentInfo(directDebitPaymentInfoModel);
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(any(PaymentInfoModel.class))).thenReturn(pspDirectDebitPaymentInfoConvertStrategy);

        // When
        pspCheckoutFacade.getPaymentDetails();

        // Then
        verify(pspDirectDebitPaymentInfoConvertStrategy).convert(directDebitPaymentInfoModel);
    }

    @Test
    public void savePayPalSubscription() {
        // Given
        PspPaymentModeData pspPaymentModeData = setupPaymentModeData();
        when(pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel)).thenReturn(pspPayPalPaymentInfoCreateStrategy);

        // When
        pspCheckoutFacade.saveSubscription(pspPaymentModeData);

        //Then
        verify(pspPayPalPaymentInfoCreateStrategy).saveSubscription(cartModel);
    }

    @Test
    public void authorizeCardPaymentForUncertain() {
        // Given
        paymentTransactionEntryModel.setTransactionStatus(PspTransactionStatus.UNCERTAIN.name());

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.UNCERTAIN, result.getResult());
    }

    @Test
    public void authorizeCardPaymentForNullTransactionEntry() {
        // Given that authorise returns null
        when(commerceCheckoutService.authorizePayment(any(CommerceCheckoutParameter.class))).thenReturn(null);

        // When
        PspCardAuthorizationData result = pspCheckoutFacade.authorizeCardPayment(securityCode);

        // Then
        assertEquals(PspAuthResult.FAILURE, result.getResult());
    }


    private PspPaymentModeData setupPaymentModeData() {
        PspPaymentModeData pspPaymentModeData = new PspPaymentModeData();
        List<PspPaymentMode> pspPaymentModeList = new ArrayList<>();
        PspPaymentMode pspPaymentMode = new PspPaymentMode();
        List<PspBrand> brandList = new ArrayList<>();
        PspBrand pspBrand = new PspBrand();
        pspBrand.setComposedTypeModel(composedTypeModel);
        brandList.add(pspBrand);
        pspPaymentMode.setBrands(brandList);
        pspPaymentModeList.add(pspPaymentMode);
        pspPaymentModeData.setPaymentModes(pspPaymentModeList);

        return pspPaymentModeData;
    }

    @Test
    public void setPaymentModeIfAvailable() {
        // Given
        cartModel.setPaymentMode(null);
        PaymentModeModel paymentModeModel = getPaymentModeModel();
        when(paymentModeService.getAvailablePaymentMethods()).thenReturn(Arrays.asList(paymentModeModel));

        // When
        pspCheckoutFacade.setPaymentModeIfAvailable();

        // Then
        assertNotNull(cartModel.getPaymentMode());

    }

    private PaymentModeModel getPaymentModeModel() {
        PaymentModeModel paymentModeModel = new PaymentModeModel();
        paymentModeModel.setCode(DEFAULT_PAYMENT_MODE);
        paymentModeModel.setActive(Boolean.TRUE);
        return paymentModeModel;
    }

    @Test
    public void setPaymentMode() {
        // Given
        PaymentModeModel paymentModeModel = getPaymentModeModel();
        when(paymentModeService.getAvailablePaymentModeForCode(anyString())).thenReturn(Arrays.asList(paymentModeModel));

        // When
        pspCheckoutFacade.setPaymentMode(DEFAULT_PAYMENT_MODE);

        // Then
        assertNotNull(cartModel.getPaymentMode());

    }

    @Test(expected = IllegalArgumentException.class)
    public void setPaymentDetailsShouldthrowException() {
        // Given no input

        // When
        pspCheckoutFacade.setPaymentDetails(null);

        // Then throws an exception
    }

    @Test
    public void setPaymentDetailsNoPaymentInfo() {
        // When
        pspCheckoutFacade.setPaymentDetails("id");

        // Then
        verify(customerAccountService).getPaymentInfoForCode(userModel, "id");
        verify(commerceCheckoutService, never()).setPaymentInfo(any(CommerceCheckoutParameter.class));
    }

    @Test
    public void setPaymentDetailsWithPaymentInfo() {
        // Given
        when(customerAccountService.getPaymentInfoForCode(any(CustomerModel.class), anyString())).thenReturn(paymentInfoModel);

        // When
        pspCheckoutFacade.setPaymentDetails("id");

        // Then
        verify(commerceCheckoutService).setPaymentInfo(any(CommerceCheckoutParameter.class));

    }

    @Test
    public void setPaymentDetailsWithNoUserMatching() {
        // Given
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(null);

        // When
        pspCheckoutFacade.setPaymentDetails("id");

        // Then
        verify(customerAccountService, never()).getPaymentInfoForCode(userModel, "id");
    }

    @Test
    public void setPaymentDetailsWithBlankParameter() {
        // When
        pspCheckoutFacade.setPaymentDetails("");

        // Then
        verify(customerAccountService, never()).getPaymentInfoForCode(userModel, "id");
    }

    @Test
    public void codeFetchedIsFromErrorCodeField() {
        // Given
        String paymentErrorCode = "paymentErrorCode";
        cartModel.setPaymentErrorCode(paymentErrorCode);

        // When
        String result = pspCheckoutFacade.fetchPaymentErrorCode();

        // Then
        assertEquals(paymentErrorCode, result);
    }

    @Test
    public void blankErrorCodeIsFetched() {
        // Given
        cartModel.setPaymentErrorCode(StringUtils.EMPTY);
        cartModel.setPaymentErrorStatus(StringUtils.EMPTY);
        // When
        String result = pspCheckoutFacade.fetchPaymentErrorCode();

        // Then
        assertEquals(StringUtils.EMPTY, result);
    }

    @Test
    public void codeFetchedIsFromErrorStatusField() {
        // Given
        cartModel.setPaymentErrorCode(StringUtils.EMPTY);
        String paymentErrorStatus = "PaymentErrorStatus";
        cartModel.setPaymentErrorStatus(paymentErrorStatus);

        // When
        String result = pspCheckoutFacade.fetchPaymentErrorCode();

        // Then
        assertEquals(paymentErrorStatus, result);
    }

    private void populateCartWithPaymentInfo() {
        paymentInfoModel = new CreditCardPaymentInfoModel();
        paymentInfoModel.setSubscriptionId(SUBSCRIPTION_ID);
        cartModel = new CartModel();
        cartModel.setPaymentInfo(paymentInfoModel);
    }

    private void setupMockingBehaviour() {
        when(cartFacade.hasSessionCart()).thenReturn(Boolean.TRUE);
        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(userModel);
        when(commerceCheckoutService.authorizePayment(any(CommerceCheckoutParameter.class))).thenReturn(paymentTransactionEntryModel);
    }

    private void populateUser() {
        userModel = new CustomerModel();
        userModel.setUid("uniqueUserId");
        cartModel.setUser(userModel);
    }
}
