package com.psp.payment.common.services.impl;

import com.psp.model.ErrorMessageModel;
import com.psp.model.StatusMessageModel;
import com.psp.payment.common.dao.PspErrorMessageDao;
import com.psp.payment.common.dao.PspStatusMessageDao;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspErrorMessageServiceImplTest {

    private final String TEST_ERROR_CODE = "errorcode";
    private final String TEST_ERROR_MESSAGE = "Test Error Message";
    private final String TEST_STATUS_CODE = "statuscode";
    private final String TEST_STATUS_DESCRIPTION = "Test Status Message";

    @InjectMocks
    private PspErrorMessageServiceImpl pspErrorMessageService;

    @Mock
    private PspErrorMessageDao pspErrorMessageDao;

    @Mock
    private PspStatusMessageDao pspStatusMessageDao;

    @Mock
    private ErrorMessageModel errorMessageModel;

    @Mock
    private StatusMessageModel statusMessageModel;

    private List<ErrorMessageModel> errorMessageModelList;

    private List<StatusMessageModel> statusMessageModelList;

    @Before
    public void setup() {
        errorMessageModelList = new ArrayList<>();
        errorMessageModelList.add(errorMessageModel);
        statusMessageModelList = new ArrayList<>();
        statusMessageModelList.add(statusMessageModel);

        setupMockingBehaviour();
    }

    @Test(expected = IllegalArgumentException.class)
    public void errorMessageIsNotFetchedOnInvalidInputs() {
        // When
        pspErrorMessageService.fetchErrorMessageByCode(null);

        // Then an exception is thrown
    }

    @Test
    public void internalCallsAreMadeToFetchErrorMessage() {
        // When
        pspErrorMessageService.fetchErrorMessageByCode(TEST_ERROR_CODE);

        // Then
        verify(pspErrorMessageDao).findErrorMessageByCode(TEST_ERROR_CODE);
    }

    @Test
    public void errorMessageIsFetched() {
        // When
        String result = pspErrorMessageService.fetchErrorMessageByCode(TEST_ERROR_CODE);

        // Then
        assertEquals(TEST_ERROR_MESSAGE, result);
    }

    @Test(expected = IllegalStateException.class)
    public void errorMessageIsNotFetchedIfMoreThanOneMessageFound() {
        // Given
        ErrorMessageModel anotherErrorMessageModel = new ErrorMessageModel();
        anotherErrorMessageModel.setMessage(TEST_ERROR_MESSAGE);
        errorMessageModelList.add(anotherErrorMessageModel);

        // When
        pspErrorMessageService.fetchErrorMessageByCode(TEST_ERROR_CODE);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void errorMessageForStatusIsNotFetchedForInvalidInputs() {
        // When
        pspErrorMessageService.fetchErrorMessageByStatus(null);

        // Then an exception is thrown
    }

    @Test
    public void internalCallsAreMadeToFetchErrorMessageForStatusCode() {
        // When
        pspErrorMessageService.fetchErrorMessageByStatus(TEST_STATUS_CODE);

        // Then
        verify(pspStatusMessageDao).findErrorMessageByStatus(TEST_STATUS_CODE);
    }

    @Test
    public void errorMessageIsFetchedForStatusCode() {
        // When
        String result = pspErrorMessageService.fetchErrorMessageByStatus(TEST_STATUS_CODE);

        // Then
        assertEquals(TEST_STATUS_DESCRIPTION, result);
    }

    @Test(expected = IllegalStateException.class)
    public void errorMessageIsNotFetchedIfMoreThanOneMessageFoundForStatus() {
        // Given
        StatusMessageModel anotherStatusMessageModel = new StatusMessageModel();
        anotherStatusMessageModel.setDescription(TEST_STATUS_DESCRIPTION);
        statusMessageModelList.add(anotherStatusMessageModel);

        // When
        pspErrorMessageService.fetchErrorMessageByStatus(TEST_STATUS_CODE);

        // Then an exception is thrown
    }

    private void setupMockingBehaviour() {
        when(errorMessageModel.getMessage()).thenReturn(TEST_ERROR_MESSAGE);
        when(pspErrorMessageDao.findErrorMessageByCode(TEST_ERROR_CODE)).thenReturn(errorMessageModelList);
        when(statusMessageModel.getDescription()).thenReturn(TEST_STATUS_DESCRIPTION);
        when(pspStatusMessageDao.findErrorMessageByStatus(TEST_STATUS_CODE)).thenReturn(statusMessageModelList);
    }
}
