package com.psp.payment.common.services.impl;

import com.psp.enums.PspPaymentProcessOption;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.services.PspPaymentModeService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.site.BaseSiteService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentProcessServiceImplTest {

    @InjectMocks
    private PspPaymentProcessServiceImpl pspPaymentProcessService;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private PspPaymentModeService pspPaymentModeService;

    @Mock
    private BaseSiteModel baseSiteModel;

    private PaymentModeModel paymentModeModel;

    @Before
    public void setup() {
        paymentModeModel = new PaymentModeModel();
    }

    @Test
    public void forceDirectSaleEnabledGivesSalOperation() {
        // Given
        paymentModeModel.setForceDirectSale(Boolean.TRUE);

        // When
        PspOperation result = pspPaymentProcessService.findOperationForPaymentMode(paymentModeModel);

        // Then
        assertEquals(PspOperation.SAL, result);
    }

    @Test
    public void forceDirectSaleDisabledGivesSalOperation() {
        // Given
        paymentModeModel.setForceDirectSale(Boolean.FALSE);
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(baseSiteModel.getPaymentProcess()).thenReturn(PspPaymentProcessOption.DIRECTSALE);

        // When
        PspOperation result = pspPaymentProcessService.findOperationForPaymentMode(paymentModeModel);

        // Then
        assertEquals(PspOperation.SAL, result);
    }

    @Test
    public void forceDirectSaleDisabledGivesResOperation() {
        // Given
        paymentModeModel.setForceDirectSale(Boolean.FALSE);
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(baseSiteModel.getPaymentProcess()).thenReturn(PspPaymentProcessOption.AUTHORISE);

        // When
        PspOperation result = pspPaymentProcessService.findOperationForPaymentMode(paymentModeModel);

        // Then
        assertEquals(PspOperation.RES, result);
    }

    @Test(expected = IllegalArgumentException.class)
    public void operationNotGeneratedWhenPaymentModeNotProvided() {
        // When
        pspPaymentProcessService.findOperationForPaymentMode(null);

        // Then an exception is thrown
    }

    @Test
    public void captureIsReturnedIfForceDirectSaleIsEnabled() {
        // Given
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        when(pspPaymentModeService.getPaymentModeForPaymentInfo(payPalPaymentInfoModel)).thenReturn(paymentModeModel);
        paymentModeModel.setForceDirectSale(Boolean.TRUE);

        // When
        PaymentTransactionType result = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(payPalPaymentInfoModel, baseSiteModel);

        // Then
        assertEquals(PaymentTransactionType.CAPTURE, result);
    }

    @Test
    public void captureIsReturnedIfForceDirectSaleIsDisabled() {
        // Given
        paymentModeModel.setForceDirectSale(Boolean.FALSE);
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        when(pspPaymentModeService.getPaymentModeForPaymentInfo(payPalPaymentInfoModel)).thenReturn(paymentModeModel);
        when(baseSiteModel.getPaymentProcess()).thenReturn(PspPaymentProcessOption.DIRECTSALE);

        // When
        PaymentTransactionType result = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(payPalPaymentInfoModel, baseSiteModel);

        // Then
        assertEquals(PaymentTransactionType.CAPTURE, result);
    }

    @Test
    public void authoriseIsReturnedIfForceDirectSaleIsDisabled() {
        // Given
        paymentModeModel.setForceDirectSale(Boolean.FALSE);
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        when(pspPaymentModeService.getPaymentModeForPaymentInfo(payPalPaymentInfoModel)).thenReturn(paymentModeModel);
        when(baseSiteModel.getPaymentProcess()).thenReturn(PspPaymentProcessOption.AUTHORISE);

        // When
        PaymentTransactionType result = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(payPalPaymentInfoModel, baseSiteModel);

        // Then
        assertEquals(PaymentTransactionType.AUTHORIZATION, result);
    }

    @Test(expected = IllegalArgumentException.class)
    public void paymentTransactionTypeIsNotFoundForInvalidPaymentInfo() {
        // When
        pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(null, baseSiteModel);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void paymentTransactionTypeIsNotFoundForInvalidBaseSite() {
        // Given
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();

        // When
        pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(payPalPaymentInfoModel, null);

        // Then an exception is thrown
    }
}
