package com.psp.payment.common.services.impl;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.enums.PspPaymentProcessOption;
import com.psp.model.AbstractFlexPaymentInfoModel;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.dao.PspPaymentModeDao;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.strategies.PspPostPaymentFeedbackStrategy;
import com.psp.payment.common.strategies.PspSignatureValidationStrategy;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.dto.PspAuthorizationResult;
import com.psp.payment.directlink.dto.PspSubscriptionAuthorizationRequest;
import com.psp.payment.flexpayments.strategies.PspFlexPaymentInfoCreateStrategy;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import com.psp.payment.tokenisation.factories.PspResponseStrategyFactory;
import com.psp.payment.tokenisation.strategies.PspCreateSubscriptionRequestStrategy;
import com.psp.payment.tokenisation.strategies.PspPaymentResponseInterpretationStrategy;
import com.psp.payment.tokenisation.strategies.impl.PspErrorResponseStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspPaymentResponseInterpretationStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspSucessResponseStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.commerceservices.strategies.GenerateMerchantTransactionCodeStrategy;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.methods.CardPaymentService;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.site.BaseSiteService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Matchers;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import static org.junit.Assert.*;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentServiceImplTest {

    private static final BigDecimal AMOUNT = BigDecimal.valueOf(10);
    private static final String CURRENCY_CODE = "GBP";
    private static final String UNIQUE_ORDER_ID = "ORDER-1";
    private static final String PAY_ID = "pay-id";
    private static final String ACCEPTANCE = "test123";
    private static final String MERCHANT_CODE = "uuid";
    private static final String MERCHANT_TRANSACTION_CODE = "173412-TXN-CODE-43142356";
    private static final String CUSTOMER_ID = "1000";

    @InjectMocks
    private PspPaymentServiceImpl pspPaymentService;

    @Mock
    private PspCreateSubscriptionRequestStrategy pspCreateSubscriptionRequestStrategy;

    @Mock
    private Converter<PspCreateSubscriptionRequest, PaymentData> pspPaymentDataConverter;

    @Mock
    private CartService cartService;

    @Mock
    private ModelService modelService;

    @Mock
    private PspPaymentResponseInterpretationStrategyImpl paymentResponseInterpretationStrategy;

    @Mock
    private CardPaymentService cardPaymentService;

    @Mock
    private CommonI18NService commonI18NService;

    @Mock
    private PspUtils pspUtils;

    @Mock
    private PspSignatureValidationStrategy pspSignatureValidationStrategy;

    @Mock
    private PspPaymentResponseInterpretationStrategy pspPaymentResponseInterpretationStrategy;

    @Mock
    private PspResponseStrategyFactory pspResponseStrategyFactory;

    @Mock
    private PspSucessResponseStrategyImpl pspSucessResponseStrategy;

    @Mock
    private PspErrorResponseStrategyImpl pspErrorResponseStrategy;

    @Mock
    private PspPaymentModeDao pspPaymentModeDao;

    private PspAuthorizationResult pspAuthorizationResult;

    private CustomerModel customerModel;

    private CartModel cartModel;

    private PayPalPaymentInfoModel payPalPaymentInfoModel;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private GenerateMerchantTransactionCodeStrategy generateMerchantTransactionCodeStrategy;

    @Mock
    private PspPostPaymentFeedbackStrategy pspPostPaymentFeedbackStrategy;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Mock
    private PspFlexPaymentInfoCreateStrategy pspFlexPaymentInfoCreateStrategy;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    private PaymentModeModel paymentModeModel;

    @Before
    public void setUp() {
        customerModel = new CustomerModel();
        customerModel.setCustomerID(CUSTOMER_ID);
        cartModel = new CartModel();
        paymentModeModel = new PaymentModeModel();
        ComposedTypeModel composedTypeModel = new ComposedTypeModel();
        composedTypeModel.setCode("PayPalPaymentInfo");
        paymentModeModel.setPaymentInfoType(composedTypeModel);
        pspAuthorizationResult = new PspAuthorizationResult();
        payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        BaseSiteModel baseSiteModel = new BaseSiteModel();
        baseSiteModel.setPaymentProcess(PspPaymentProcessOption.DIRECTSALE);
        cartModel.setSite(baseSiteModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel)).thenReturn(pspFlexPaymentInfoCreateStrategy);
        when(pspFlexPaymentInfoCreateStrategy.saveSubscription(cartModel)).thenReturn(payPalPaymentInfoModel);
    }

    @Test
    public void testBeginHopCreatePaymentSubscriptionIsValid() {
        // Given
        PspCreateSubscriptionRequest pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        when(pspCreateSubscriptionRequestStrategy.createSubscriptionRequest(cartModel, customerModel)).thenReturn(pspCreateSubscriptionRequest);
        when(pspPaymentDataConverter.convert(pspCreateSubscriptionRequest)).thenReturn(new PaymentData());

        // When
        pspPaymentService.beginHopCreatePaymentSubscription(null, null, null, customerModel, null, null);

        // Then
        verify(cartService).getSessionCart();
        verify(pspCreateSubscriptionRequestStrategy).createSubscriptionRequest(cartModel, customerModel);
        verify(pspPaymentDataConverter).convert(pspCreateSubscriptionRequest);
    }

    @Test
    public void successfullyCompleteHopCreatePaymentSubscription() {
        // Given
        cartModel.setCode("1234");
        PspPaymentSubscriptionResultItem pspPaymentSubscriptionResultItem = new PspPaymentSubscriptionResultItem();
        PspCreateSubscriptionResult pspCreateSubscriptionResult = new PspCreateSubscriptionResult();
        Map<String, String> parameters = new TreeMap<>();
        parameters.put(PspConstants.KEY_RESULT_ORDERID, "1234-1");

        when(pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)).thenReturn(Boolean.TRUE);
        when(pspPaymentResponseInterpretationStrategy.interpretResponse(parameters)).thenReturn(pspCreateSubscriptionResult);
        when(pspResponseStrategyFactory.getResponseStrategy(pspCreateSubscriptionResult)).thenReturn(pspSucessResponseStrategy);
        when(pspSucessResponseStrategy.handleResponse(pspCreateSubscriptionResult)).thenReturn(pspPaymentSubscriptionResultItem);

        // When
        PspPaymentSubscriptionResultItem result = pspPaymentService.completeHopCreatePaymentSubscription(parameters);

        // Then
        assertEquals(pspPaymentSubscriptionResultItem, result);
        verify(pspSignatureValidationStrategy).validateSignature(parameters, PspConstants.KEY_SHASIGN);
        verify(pspPaymentResponseInterpretationStrategy).interpretResponse(parameters);
        verify(pspResponseStrategyFactory).getResponseStrategy(pspCreateSubscriptionResult);
        verify(pspSucessResponseStrategy).handleResponse(pspCreateSubscriptionResult);
    }

    @Test
    public void createPaymentSubscriptionFailsWithShaSignValidation() {
        // Given
        Map<String, String> parameters = new TreeMap<>();
        when(pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)).thenReturn(Boolean.FALSE);

        // When
        PspPaymentSubscriptionResultItem result = pspPaymentService.completeHopCreatePaymentSubscription(parameters);

        // Then
        assertNull(result);
        verify(pspSignatureValidationStrategy).validateSignature(parameters, PspConstants.KEY_SHASIGN);
        verify(pspPaymentResponseInterpretationStrategy, never()).interpretResponse(parameters);
        verify(pspResponseStrategyFactory, never()).getResponseStrategy(any(PspCreateSubscriptionResult.class));
        verify(pspSucessResponseStrategy, never()).handleResponse(any(PspCreateSubscriptionResult.class));
    }

    @Test
    public void createPaymentSubscriptionFailsWithErrorResponse() {
        // Given
        cartModel.setCode("1234");
        PspPaymentSubscriptionResultItem pspPaymentSubscriptionResultItem = new PspPaymentSubscriptionResultItem();
        PspCreateSubscriptionResult pspCreateSubscriptionResult = new PspCreateSubscriptionResult();
        Map<String, String> parameters = new TreeMap<>();
        parameters.put(PspConstants.KEY_RESULT_ORDERID, "1234-1");

        when(pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)).thenReturn(Boolean.TRUE);
        when(pspPaymentResponseInterpretationStrategy.interpretResponse(parameters)).thenReturn(pspCreateSubscriptionResult);
        when(pspResponseStrategyFactory.getResponseStrategy(pspCreateSubscriptionResult)).thenReturn(pspErrorResponseStrategy);
        when(pspErrorResponseStrategy.handleResponse(pspCreateSubscriptionResult)).thenReturn(pspPaymentSubscriptionResultItem);

        // When
        PspPaymentSubscriptionResultItem result = pspPaymentService.completeHopCreatePaymentSubscription(parameters);

        // Then
        assertEquals(pspPaymentSubscriptionResultItem, result);
        verify(pspSignatureValidationStrategy).validateSignature(parameters, PspConstants.KEY_SHASIGN);
        verify(pspPaymentResponseInterpretationStrategy).interpretResponse(parameters);
        verify(pspResponseStrategyFactory).getResponseStrategy(pspCreateSubscriptionResult);
        verify(pspErrorResponseStrategy).handleResponse(pspCreateSubscriptionResult);
    }

    @Test
    public void testAuthorization() {
        // Given
        BaseSiteModel baseSiteModel = new BaseSiteModel();
        cartModel.setSite(baseSiteModel);
        populateAuthorisationResultForSuccess();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        PaymentTransactionEntryModel entry = new PaymentTransactionEntryModel();
        CurrencyModel currencyModel = new CurrencyModel();
        currencyModel.setIsocode(CURRENCY_CODE);
        Currency currency = getCurrency();

        when(modelService.create(PaymentTransactionModel.class)).thenReturn(paymentTransactionModel);
        when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(entry);
        when(cardPaymentService.authorize(any(PspSubscriptionAuthorizationRequest.class))).thenReturn(pspAuthorizationResult);
        when(commonI18NService.getCurrency(CURRENCY_CODE)).thenReturn(currencyModel);
        when(pspUtils.generateUniqueOrderId(cartModel)).thenReturn(UNIQUE_ORDER_ID);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(), eq(baseSiteModel))).thenReturn(PaymentTransactionType.CAPTURE);

        // When
        final PaymentTransactionEntryModel paymentTransactionEntryModel = pspPaymentService.authorize(MERCHANT_TRANSACTION_CODE,
                AMOUNT, currency, new AddressModel(), StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);

        // Then
        assertEquals(AMOUNT.longValue(), paymentTransactionEntryModel.getAmount().longValue());
        assertNotNull(paymentTransactionEntryModel.getPaymentTransaction());
        assertEquals(TransactionStatus.ACCEPTED.name(), paymentTransactionEntryModel.getTransactionStatus());
        assertEquals(TransactionStatusDetails.SUCCESFULL.name(), paymentTransactionEntryModel.getTransactionStatusDetails());
        assertEquals(PaymentTransactionType.CAPTURE.name(), paymentTransactionEntryModel.getType().getCode());
    }

    @Test
    public void testFailedAuthorization() {
        // Given
        BaseSiteModel baseSiteModel = new BaseSiteModel();
        cartModel.setSite(baseSiteModel);
        populateAuthorisationResultForFailure();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        PaymentTransactionEntryModel entry = new PaymentTransactionEntryModel();
        final CurrencyModel currencyModel = new CurrencyModel();
        currencyModel.setIsocode(CURRENCY_CODE);
        final Currency currency = getCurrency();

        when(commonI18NService.getCurrency(CURRENCY_CODE)).thenReturn(currencyModel);
        when(modelService.create(PaymentTransactionModel.class)).thenReturn(paymentTransactionModel);
        when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(entry);
        when(cardPaymentService.authorize(Matchers.any(PspSubscriptionAuthorizationRequest.class))).thenReturn(pspAuthorizationResult);
        when(pspUtils.generateUniqueOrderId(cartModel)).thenReturn(UNIQUE_ORDER_ID);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(), eq(baseSiteModel))).thenReturn(PaymentTransactionType.CAPTURE);

        // When
        final PaymentTransactionEntryModel paymentTransactionEntryModel = pspPaymentService.authorize(MERCHANT_TRANSACTION_CODE,
                AMOUNT, modelService.toPersistenceLayer(currency), new AddressModel(), StringUtils.EMPTY, StringUtils.EMPTY,
                StringUtils.EMPTY);

        // Then
        assertEquals(TransactionStatus.REJECTED.name(), paymentTransactionEntryModel.getTransactionStatus());
        assertEquals(TransactionStatusDetails.CARD_USED_TOO_RECENTLY.name(), paymentTransactionEntryModel.getTransactionStatusDetails());
        assertEquals(PaymentTransactionType.CAPTURE.name(), paymentTransactionEntryModel.getType().getCode());
    }

    @Test
    public void sessionCartNumberMatchesResponse() {
        // Given
        cartModel.setCode("00000001");
        cartModel.setTotalPrice(10.5);
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_RESULT_ORDERID, "00000001-1");
        parameters.put(PspConstants.KEY_RESULT_AMOUNT, "10.5");

        // When
        boolean result = pspPaymentService.validateCartDetails(parameters, PspConstants.KEY_RESULT_ORDERID, PspConstants.KEY_RESULT_AMOUNT);

        // Then
        assertTrue(result);
    }

    @Test
    public void sessionCartNotMatchingResponse() {
        // Given
        cartModel.setCode("0");
        cartModel.setTotalPrice(0.0);
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_RESULT_ORDERID, "00000001-1");

        // When
        boolean result = pspPaymentService.validateCartDetails(parameters, PspConstants.KEY_RESULT_ORDERID, PspConstants.KEY_RESULT_AMOUNT);

        // Then
        assertFalse(result);
    }

    @Test
    public void sessionCartNotMatchingEmptyResponse() {
        // Given
        cartModel.setCode("00000001-1");
        cartModel.setTotalPrice(10.0);
        Map<String, String> parameters = new HashMap<>();

        // When
        boolean result = pspPaymentService.validateCartDetails(parameters, PspConstants.KEY_RESULT_ORDERID, PspConstants.KEY_RESULT_AMOUNT);

        // Then
        assertFalse(result);
    }

    private Currency getCurrency() {
        return Currency.getInstance(CURRENCY_CODE);
    }

    private void populateAuthorisationResultForSuccess() {
        pspAuthorizationResult.setTotalAmount(AMOUNT);
        pspAuthorizationResult.setPspTransactionStatus(TransactionStatus.ACCEPTED.name());
        pspAuthorizationResult.setTransactionStatusDetails(TransactionStatusDetails.SUCCESFULL);
    }

    private void populateAuthorisationResultForFailure() {
        pspAuthorizationResult.setTotalAmount(AMOUNT);
        pspAuthorizationResult.setPspTransactionStatus(TransactionStatus.REJECTED.name());
        pspAuthorizationResult.setTransactionStatusDetails(TransactionStatusDetails.CARD_USED_TOO_RECENTLY);
    }

    @Test(expected = IllegalArgumentException.class)
    public void createPaymentInfoNullParams() {
        pspPaymentService.createPaymentInfo(null);
    }

    @Test
    public void createPaymentInfoValidParams() {
        // Given
        cartModel.setPaymentMode(paymentModeModel);
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_PAYID, PAY_ID);
        parameters.put(PspConstants.KEY_RESULT_ACCEPTANCE, ACCEPTANCE);
        when(modelService.create(PayPalPaymentInfoModel.class)).thenReturn(payPalPaymentInfoModel);
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(customerModel);
        when(generateMerchantTransactionCodeStrategy.generateCode(cartModel)).thenReturn(MERCHANT_CODE);
        // When
        AbstractFlexPaymentInfoModel payPalPaymentInfoModel = pspPaymentService.createPaymentInfo(parameters);

        //Then
        assertEquals(PAY_ID, payPalPaymentInfoModel.getPayId());
        verify(modelService).saveAll(payPalPaymentInfoModel, cartModel);

    }

    @Test
    public void processPostPaymentFeedbackWithInvalidSignature() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        when(pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)).thenReturn(Boolean.FALSE);

        //When
        pspPaymentService.processPostPaymentFeedback(parameters);

        //Then
        verify(pspPostPaymentFeedbackStrategy, never()).handleFeedback(parameters);

    }

    @Test
    public void processPostPaymentFeedbackWithValidSignature() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, "9");
        when(pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_SHASIGN)).thenReturn(Boolean.TRUE);

        //When
        pspPaymentService.processPostPaymentFeedback(parameters);

        //Then
        verify(pspPostPaymentFeedbackStrategy, atLeastOnce()).handleFeedback(parameters);

    }

    @Test(expected = UnsupportedOperationException.class)
    public void beginSopSubscriptionIsNotSupported() {
        // When
        pspPaymentService.beginSopCreatePaymentSubscription(null, null, null, null, null, null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void completeHopSubscriptionIsNotSupported() {
        // When
        pspPaymentService.completeHopCreatePaymentSubscription(null, Boolean.FALSE, null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void completeSopSubscriptionIsNotSupported() {
        // When
        pspPaymentService.completeSopCreatePaymentSubscription(null, Boolean.FALSE, null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void handleCreateSubscriptionCallbackIsNotSupported() {
        // When
        pspPaymentService.handleCreateSubscriptionCallback(null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void handleFraudUpdateCallbackIsNotSupported() {
        // When
        pspPaymentService.handleFraudUpdateCallback(null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setPaymentTransactionReviewResultIsNotSupported() {
        // When
        pspPaymentService.setPaymentTransactionReviewResult(null, null);
    }
}
