package com.psp.payment.common.strategies.impl;

import com.psp.enums.PspTransactionStatus;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.services.impl.PspPaymentServiceImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.commerceservices.strategies.GenerateMerchantTransactionCodeStrategy;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.i18n.I18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Currency;

import static org.junit.Assert.assertNull;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCommercePaymentAuthorizationStrategyImplTest {
    private static final String PAY_ID = "99999999";
    private final String PAYMENT_PROVIDER = "psp";
    private final String CVV_CODE = "123";
    private final int AUTH_AMOUNT = 50;

    @InjectMocks
    private PspCommercePaymentAuthorizationStrategyImpl pspCommercePaymentAuthorizationStrategy;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private CartModel cartModel;

    @Mock
    private AddressModel deliveryAddress;

    private PaymentTransactionModel paymentTransactionModel;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private GenerateMerchantTransactionCodeStrategy generateMerchantTransactionCodeStrategy;

    @Mock
    private PspPaymentServiceImpl pspPaymentService;

    @Mock
    private I18NService i18nService;

    @Mock
    private CreditCardPaymentInfoModel paymentInfoModel;

    @Mock
    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;

    @Mock
    private PayPalPaymentInfoModel paypalPaymentInfoModel;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private CurrencyModel currencyModel;

    @Mock
    private ModelService modelService;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private CommerceCheckoutParameter parameter;

    @Before
    public void setUp() {
        parameter = getParamter();
        when(cartModel.getPaymentInfo()).thenReturn(paymentInfoModel);
        when(cartModel.getCurrency()).thenReturn(currencyModel);
        when(paymentInfoModel.getSubscriptionId()).thenReturn(CVV_CODE);
        pspCommercePaymentAuthorizationStrategy.setI18nService(i18nService);
        pspCommercePaymentAuthorizationStrategy.setPaymentService(pspPaymentService);
        pspCommercePaymentAuthorizationStrategy.setModelService(modelService);
        pspCommercePaymentAuthorizationStrategy.setGenerateMerchantTransactionCodeStrategy(generateMerchantTransactionCodeStrategy);


    }

    @Test
    public void authorizePaymentAmount() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PspTransactionStatus.UNCERTAIN.name());
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(paymentTransactionEntryModel);

        // When
        pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        verify(modelService).saveAll(cartModel, paymentTransactionModel);

    }
 @Test
    public void authorizePaymentAmountDirectDebits() throws Exception {
        // Given
        when(cartModel.getPaymentInfo()).thenReturn(directDebitPaymentInfoModel);
        when(directDebitPaymentInfoModel.getSubscriptionId()).thenReturn(CVV_CODE);
        paymentTransactionModel = getTransaction(PspTransactionStatus.UNCERTAIN.name());
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(paymentTransactionEntryModel);

        // When
        pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        verify(modelService).saveAll(cartModel, paymentTransactionModel);

    }

    @Test
    public void authorizePaymentAmountRejectedTransaction() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PspTransactionStatus.REJECTED.name());
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(paymentTransactionEntryModel);

        // When
        pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        verify(modelService).removeAll(Arrays.asList(paymentTransactionModel, paymentTransactionEntryModel));

    }

    @Test
    public void authorizePaymentAmountFailedAuth() throws Exception {
        // Given
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(null);

        // When
        PaymentTransactionEntryModel transactionEntryModel = pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        assertNull(transactionEntryModel);

    }

    @Test
    public void authorizePaymentAmountWithAcceptedTransaction() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PspTransactionStatus.ACCEPTED.name());
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(paymentTransactionEntryModel);

        // When
        pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        verify(modelService).saveAll(cartModel, paymentTransactionModel);

    }

    @Test
    public void authorizePaymentAmountWithTransactionInReview() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PspTransactionStatus.REVIEW.name());
        when(pspPaymentService.authorize(anyString(), (BigDecimal) any(BigDecimal.class), (Currency) any(Currency.class), (AddressModel) any(AddressModel.class), anyString(), anyString(), anyString())).thenReturn(paymentTransactionEntryModel);

        // When
        pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        verify(modelService).saveAll(cartModel, paymentTransactionModel);

    }

    @Test
    public void authorizePaymentAmountWithNoCreditCard() throws Exception {
        // Given
        when(cartModel.getPaymentInfo()).thenReturn(paypalPaymentInfoModel);
        paymentTransactionModel = getTransaction(PspTransactionStatus.REVIEW.name());

        // When
        PaymentTransactionEntryModel transactionEntryModel = pspCommercePaymentAuthorizationStrategy.authorizePaymentAmount(parameter);

        //Then
        assertNull(transactionEntryModel);

    }

    private PaymentTransactionModel getTransaction(String status) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionModel.setRequestId(PAY_ID);
        paymentTransactionModel.setRequestToken(PAY_ID);

        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(PaymentTransactionType.CAPTURE);
        paymentTransactionEntryModel.setTransactionStatus(status);
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));

        return paymentTransactionModel;
    }


    private CommerceCheckoutParameter getParamter() {
        CommerceCheckoutParameter parameter = new CommerceCheckoutParameter();
        parameter.setCart(cartModel);
        parameter.setEnableHooks(true);
        parameter.setAuthorizationAmount(new BigDecimal(AUTH_AMOUNT));
        parameter.setSecurityCode(CVV_CODE);
        parameter.setPaymentProvider(PAYMENT_PROVIDER);
        parameter.setPaymentInfo(paymentInfoModel);
        return parameter;
    }


}