package com.psp.payment.common.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.directlink.services.PspDirectLinkService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.processengine.BusinessProcessService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;
import java.util.Map;
import java.util.TreeMap;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPostPaymentFeedbackStrategyImplTest {

    private static final String ORDER_ID = "ORDER-1";
    private static final String AMOUNT = "10";
    private static final String CURRENCY_CODE = "GBP";
    private static final String NC_ERROR = "0";
    private static final String PAYMENT_STATUS_CODE = "9";
    private static final String AUTH_STATUS_CODE = "5";
    private static final String PAY_ID = "99999999";
    private static final String FAIL_STAUTS_CODE = "2";
    private final String PROCESS_CODE = "123_ReviewDecision";

    Map<String, String> parameters;

    @InjectMocks
    private PspPostPaymentFeedbackStrategyImpl pspPostPaymentFeedbackStrategy;

    @Mock
    private BusinessProcessService businessProcessService;

    @Mock
    private PspDirectLinkService pspDirectLinkService;

    @Mock
    private ModelService modelService;

    @Mock
    private FlexibleSearchService flexibleSearchService;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private PaymentTransactionModel paymentTransactionModel;

    @Before
    public void setUp() throws Exception {
        parameters = getResponseParameters(PAYMENT_STATUS_CODE);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(PaymentTransactionType.AUTHORIZATION);

    }

    @Test
    public void handleFeedbackForOrderInUncertain() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PaymentTransactionType.AUTHORIZATION, PspTransactionStatus.UNCERTAIN.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(PspTransactionStatus.UNCERTAIN.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(businessProcessService, atLeastOnce()).triggerEvent(PROCESS_CODE);
    }

    @Test
    public void handleFeedbackForOrderNotInAuth() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PaymentTransactionType.CAPTURE, PspTransactionStatus.UNCERTAIN.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(PspTransactionStatus.UNCERTAIN.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(businessProcessService, never()).triggerEvent(PROCESS_CODE);
    }


    @Test
    public void handleFeedbackForOrderNotInUncertain() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PaymentTransactionType.AUTHORIZATION, PspTransactionStatus.ACCEPTED.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(TransactionStatus.REVIEW.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(businessProcessService, never()).triggerEvent(PROCESS_CODE);
    }

    @Test
    public void handleFeedbackForOrderNotInCapture() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(PaymentTransactionType.AUTHORIZATION, PspTransactionStatus.ACCEPTED.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(TransactionStatus.REVIEW.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(businessProcessService, never()).triggerEvent(PROCESS_CODE);
    }

    @Test
    public void handleFeedbackForOrderInAuthStatus() throws Exception {
        // Given
        parameters = getResponseParameters(AUTH_STATUS_CODE);
        paymentTransactionModel = getTransaction(PaymentTransactionType.AUTHORIZATION, PspTransactionStatus.ACCEPTED.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(PspTransactionStatus.ACCEPTED.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(businessProcessService, never()).triggerEvent(PROCESS_CODE);
    }

    @Test
    public void handleFeedbackForOrderFailedStatus() throws Exception {
        // Given
        parameters = getResponseParameters(FAIL_STAUTS_CODE);
        paymentTransactionModel = getTransaction(PaymentTransactionType.AUTHORIZATION, PspTransactionStatus.UNCERTAIN.name());

        OrderModel order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        when(flexibleSearchService.getModelByExample(any(PaymentTransactionModel.class))).thenReturn(paymentTransactionModel);
        when(pspDirectLinkService.getTransactionStatus(anyMap())).thenReturn(TransactionStatus.REJECTED.name());

        // When
        pspPostPaymentFeedbackStrategy.handleFeedback(parameters);

        //Then
        verify(modelService, never()).save(order);
    }


    private PaymentTransactionModel getTransaction(PaymentTransactionType transactionType, String status) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionModel.setRequestId(PAY_ID);
        paymentTransactionModel.setRequestToken(PAY_ID);

        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(transactionType);
        paymentTransactionEntryModel.setTransactionStatus(status);
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));

        return paymentTransactionModel;
    }


    private Map<String, String> getResponseParameters(String statusCode) {
        Map<String, String> parameters = new TreeMap<>();
        parameters.put(PspConstants.KEY_DL_ORDERID, ORDER_ID);
        parameters.put((PspConstants.KEY_RESULT_AMOUNT), AMOUNT);
        parameters.put((PspConstants.KEY_CURRENCY), CURRENCY_CODE);
        parameters.put(PspConstants.KEY_NCERROR, NC_ERROR);
        parameters.put(PspConstants.KEY_STATUS, statusCode);
        parameters.put(PspConstants.KEY_PAYID, PAY_ID);

        return parameters;
    }


}