package com.psp.payment.common.utils;

import com.psp.constants.PspConstants;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConversionException;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.BDDMockito;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.TreeMap;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspUtilsTest {

    @Spy
    @InjectMocks
    private PspUtils pspUtils;

    @Mock
    private Configuration configuration;

    @Mock
    private CartModel cartModel;

    private String secretKey = "Mysecretsig1875!?";

    @Before
    public void setup() {
        doReturn(configuration).when(pspUtils).getConfiguration();
    }

    @Test
    public void testGeneratedChecksumIsValid() {
        // Given
        Map<String, String> parameters = new TreeMap<>();
        parameters.put("AMOUNT", "1500");
        parameters.put("CURRENCY", "EUR");
        parameters.put("LANGUAGE", "en_US");
        parameters.put("ORDERID", "1234");
        parameters.put("PSPID", "MyPSPID");

        // When
        String result = pspUtils.generateChecksum(parameters, secretKey);

        // Then
        String expectedResult = "E019359BAA3456AE5A986B6AABD22CF1B3E09438739E97F17A7F61DF5A11B30F";
        assertThat(result, is(expectedResult));
    }

    @Test
    public void testGeneratedChecksumIsBlank() {
        // Given
        Map<String, String> parameters = new TreeMap<>();

        // When
        String result = pspUtils.generateChecksum(parameters, secretKey);

        // Then
        String expectedResult = "";
        assertThat(result, is(expectedResult));
    }

    @Test(expected = IllegalArgumentException.class)
    public void testGeneratedOrderIdThowsException() {
        // When
        pspUtils.generateUniqueOrderId(null);
    }

    @Test
    public void testGeneratedOrderIdIsValid() {
        // Given
        when(cartModel.getCode()).thenReturn("00000001");

        // When
        String result = pspUtils.generateUniqueOrderId(cartModel);

        // Then
        assertTrue(result.startsWith("00000001-"));
    }

    @Test
    public void testIs3DSecureDisabled() {
        // Given
        when(configuration.getBoolean(PspConstants.PARAM_3D_SECURE_ENABLED)).thenReturn(Boolean.FALSE);

        // When
        boolean is3DSecureEnabled = pspUtils.is3DSecureEnabled();

        // Then
        assertFalse(is3DSecureEnabled);
    }

    @Test
    public void testIs3DSecureEnabled() {
        // Given
        when(configuration.getBoolean(PspConstants.PARAM_3D_SECURE_ENABLED)).thenReturn(Boolean.TRUE);

        // When
        boolean is3DSecureEnabled = pspUtils.is3DSecureEnabled();

        // Then
        assertTrue(is3DSecureEnabled);
    }

    @Test
    public void testIs3DSecureEnabledNull() {
        // Given
        BDDMockito.given(configuration.getBoolean(PspConstants.PARAM_3D_SECURE_ENABLED)).willThrow(NoSuchElementException.class);

        // When
        boolean is3DSecureEnabled = pspUtils.is3DSecureEnabled();

        // Then
        assertFalse(is3DSecureEnabled);
    }

    @Test
    public void testIs3DSecureEnabledWrongValue() {
        // Given
        BDDMockito.given(configuration.getBoolean(PspConstants.PARAM_3D_SECURE_ENABLED)).willThrow(ConversionException.class);

        // When
        boolean is3DSecureEnabled = pspUtils.is3DSecureEnabled();

        // Then
        assertFalse(is3DSecureEnabled);
    }

    @Test
    public void decodeUsingBase64EncryptionSuccessfully() {
        // Given
        String encodedText = "SGVsbG8gV29ybGQ=";

        // When
        String result = pspUtils.decodeUsingBase64Encryption(encodedText);

        // Then
        String expectedResult = "Hello World";
        assertEquals(expectedResult, result);
    }

    @Test
    public void decodeUsingBase64EncryptionUnsuccessfully() {
        // Given
        String encodedText = "SGVsbG8gSGVsbG8="; // For Hello Hello

        // When
        String result = pspUtils.decodeUsingBase64Encryption(encodedText);

        // Then
        String unexpectedResult = "Hello World";
        assertNotEquals(unexpectedResult, result);
    }

    @Test
    public void decodeUsingBase64EncryptionForBlankValue() {
        // Given blank string

        // When
        String result = pspUtils.decodeUsingBase64Encryption(StringUtils.EMPTY);

        // Then
        String expectedResult = StringUtils.EMPTY;
        assertEquals(expectedResult, result);
    }

    @Test
    public void decodeUsingBase64EncryptionForNullValue() {
        // Given nothing

        // When
        String result = pspUtils.decodeUsingBase64Encryption(null);

        // Then
        String expectedResult = StringUtils.EMPTY;
        assertEquals(expectedResult, result);
    }

    @Test
    public void nonZeroAmountIsCalculatedCorrectly() {
        // Given
        BigDecimal amount = new BigDecimal("10.05");

        //When
        String result = pspUtils.getAmountMultipliedBy100(amount);

        // Then
        assertEquals("1005", result);
    }

    @Test
    public void zeroAmountIsCalculatedCorrectly() {
        // Given
        BigDecimal amount = new BigDecimal("0.00");

        //When
        String result = pspUtils.getAmountMultipliedBy100(amount);

        // Then
        assertEquals("0", result);
    }

    @Test(expected = IllegalArgumentException.class)
    public void negativeAmountIsRejected() {
        // Given
        BigDecimal amount = new BigDecimal("-1.00");

        //When
        pspUtils.getAmountMultipliedBy100(amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void noAmountIsRejected() {
        //When
        pspUtils.getAmountMultipliedBy100(null);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void websiteNameIsNotFoundForEmptyInputs() {
        // When
        String result = pspUtils.getWebsiteIdByCart(null);

        // Then an exception is thrown
    }

    @Test
    public void websiteIsNotFoundIfNotLinkedWithCart() {
        // Given
        CartModel cartModel = new CartModel();

        // When
        String result = pspUtils.getWebsiteIdByCart(cartModel);

        // Then
        assertEquals(StringUtils.EMPTY, result);
    }

    @Test
    public void websiteIdIsFoundOnCorrectInputs() {
        // Given
        final String siteUid = "apparel-uk";
        CartModel cartModel = new CartModel();
        CMSSiteModel cmsSiteModel = new CMSSiteModel();
        cmsSiteModel.setUid(siteUid);
        cartModel.setSite(cmsSiteModel);

        // When
        String result = pspUtils.getWebsiteIdByCart(cartModel);

        // Then
        assertEquals(siteUid, result);
    }
}
