package com.psp.payment.directlink.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.dto.PspSubscriptionAuthorizationRequest;
import com.psp.payment.directlink.request.PspDirectLinkRequest;
import com.psp.payment.directlink.strategies.PspAddressBuilderStrategy;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.commerceservices.i18n.CommerceCommonI18NService;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.dto.BillingInfo;
import de.hybris.platform.payment.dto.CardInfo;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.site.BaseSiteService;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Map;

import static com.psp.constants.PspConstants.*;
import static java.lang.Boolean.TRUE;
import static org.junit.Assert.*;
import static org.mockito.Answers.RETURNS_DEEP_STUBS;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectLinkRequestPopulatorTest {

    private static final String CURRENCY_CODE = "GBP";
    private static final String CARD_NUMBER = "XXXXXXXXXXXXXXXX";
    private static final String MERCHANT_TRANSACTION_CODE = "merchantTransactionCode";
    private static final String SUBSCRIPTION_ID = "subscriptionID";
    private static final String PAYMENT_PROVIDER = "paymentProvider";
    private static final String CV_2 = "cv2";
    private static final String ORDER_ID = "00006001";
    private static final int EXPIRATION_YEAR = 2018;
    private static final int EXPIRATION_MONTH = 2;
    private static final String URL = "url";
    private static final String VALUE_WIN3DS = "MAINW";
    private static final String LANGUAGE_CODE = "EN";
    private static final String PSP_ID = "test";

    @InjectMocks
    private PspDirectLinkRequestPopulator pspDirectLinkRequestPopulator;

    @Mock
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private PspUtils pspUtils;

    @Mock(answer = RETURNS_DEEP_STUBS)
    private CommerceCommonI18NService commerceCommonI18NService;

    @Mock
    private PspAddressBuilderStrategy pspAddressBuilderStrategy;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    @Mock(answer = RETURNS_DEEP_STUBS)
    private CartService cartService;

    @Mock
    private PaymentModeModel paymentModeModel;

    @Mock
    private CreditCardPaymentInfoModel creditCardPaymentInfoModel;

    @Mock(answer = RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Mock
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private BaseSiteModel baseSiteModel;

    private PspDirectLinkRequest pspDirectLinkRequest;

    private PspSubscriptionAuthorizationRequest pspSubscriptionAuthorizationRequest;

    @Before
    public void setUp() {
        pspDirectLinkRequest = new PspDirectLinkRequest();
        Currency currency = Currency.getInstance(CURRENCY_CODE);
        BillingInfo shippingInfo = new BillingInfo();
        CardInfo card = new CardInfo();
        card.setExpirationMonth(EXPIRATION_MONTH);
        card.setExpirationYear(EXPIRATION_YEAR);
        card.setCardNumber(CARD_NUMBER);

        pspSubscriptionAuthorizationRequest = new PspSubscriptionAuthorizationRequest(MERCHANT_TRANSACTION_CODE,
                SUBSCRIPTION_ID, currency, BigDecimal.ZERO, shippingInfo, CV_2, PAYMENT_PROVIDER, ORDER_ID, card);

        setupConfigurationParameters();
        setupMockingBehaviour();
    }

    @Test
    public void populate3Dsecure() {
        // Given
        when(pspUtils.is3DSecureEnabled()).thenReturn(Boolean.TRUE);

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertNotNull(pspDirectLinkRequest);
        Map<String, String> parameters = pspDirectLinkRequest.getParameters();
        assertEquals(SUBSCRIPTION_ID, parameters.get(PspConstants.KEY_ALIAS)); // Parameter present in all requests
        assertTrue(parameters.get(PspConstants.KEY_LANGUAGE).equals(LANGUAGE_CODE)); // Parameter present in 3D secure requests
    }

    @Test
    public void populateNo3Dsecure() {
        // Given
        when(pspUtils.is3DSecureEnabled()).thenReturn(Boolean.FALSE);

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertNotNull(pspDirectLinkRequest);
        Map<String, String> parameters = pspDirectLinkRequest.getParameters();
        assertEquals(SUBSCRIPTION_ID, parameters.get(PspConstants.KEY_ALIAS)); // Parameter present in all requests
        assertNull(parameters.get(PspConstants.KEY_LANGUAGE)); // Parameter present in 3D secure requests
    }

    @Test
    public void emailIsFetchedFromCustomerWhenNotPresentInAddress() {
        // Given
        CustomerModel customerModel = new CustomerModel();
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(customerModel);
        when(customerEmailResolutionService.getEmailForCustomer(customerModel)).thenReturn("test@test.com");

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertEquals("test@test.com", pspDirectLinkRequest.getParameters().get(PspConstants.KEY_EMAIL));
    }

    @Test
    public void emailIsFetchedFromAddressWhenPresent() {
        // Given
        pspSubscriptionAuthorizationRequest.getShippingInfo().setEmail("test@user.com");

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertEquals("test@user.com", pspDirectLinkRequest.getParameters().get(PspConstants.KEY_EMAIL));
    }

    @Test
    public void nameIsPopulated() {
        // Given
        when(creditCardPaymentInfoModel.getCcOwner()).thenReturn("name");

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertEquals("name", pspDirectLinkRequest.getParameters().get(PspConstants.KEY_CN));
    }

    @Test
    public void nameIsNotPopulated() {
        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertNull(pspDirectLinkRequest.getParameters().get(PspConstants.KEY_CN));

    }

    @Test
    public void trackingKeyIsPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn("HYB010317");

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertEquals("HYB010317", pspDirectLinkRequest.getParameters().get(PspConstants.KEY_TRACKING));
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn(null);

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, pspDirectLinkRequest);

        // Then
        assertNull(pspDirectLinkRequest.getParameters().get(PspConstants.KEY_TRACKING));
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateWithSourceAsNull() {
        // Given source as null

        // When
        pspDirectLinkRequestPopulator.populate(null, pspDirectLinkRequest);

        // Then an exception will be thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateWithTargetAsNull() {
        // Given target as null

        // When
        pspDirectLinkRequestPopulator.populate(pspSubscriptionAuthorizationRequest, null);

        // Then an exception will be thrown
    }

    private void setupConfigurationParameters() {
        Configuration configuration = configurationService.getConfiguration();
        when(configuration.getString(PspConstants.PARAM_PSPID)).thenReturn(PSP_ID);
        when(configuration.getString(PspConstants.PARAM_API_USER_ID)).thenReturn(PSP_ID);
        when(configuration.getString(PspConstants.PARAM_API_USER_PASSWORD)).thenReturn(PSP_ID);
        when(configuration.getString(PspConstants.PARAM_WIN3DS)).thenReturn(VALUE_WIN3DS);
        when(configuration.getString(PspConstants.PARAM_3DSECURE_ACCEPT_URL)).thenReturn(PARAM_3DSECURE_ACCEPT_URL);
        when(configuration.getString(PspConstants.PARAM_3DSECURE_DECLINE_URL)).thenReturn(PARAM_3DSECURE_DECLINE_URL);
        when(configuration.getString(PspConstants.PARAM_3DSECURE_EXCEPTION_URL)).thenReturn(PARAM_3DSECURE_EXCEPTION_URL);
    }

    private void setupMockingBehaviour() {
        when(commerceCommonI18NService.getCurrentLanguage().getIsocode()).thenReturn(LANGUAGE_CODE);
        when(pspAddressBuilderStrategy.buildAddress(anyString(), anyString())).thenReturn(StringUtils.EMPTY);
        when(pspUtils.getAmountMultipliedBy100(BigDecimal.ZERO)).thenReturn("0");
        when(cartService.getSessionCart().getPaymentMode()).thenReturn(paymentModeModel);
        when(cartService.getSessionCart().getPaymentInfo()).thenReturn(creditCardPaymentInfoModel);
        when(pspPaymentProcessService.findOperationForPaymentMode(paymentModeModel)).thenReturn(PspOperation.RES);
        when(pspUtils.getWebsiteIdByCart(cartService.getSessionCart())).thenReturn("website");
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_3DSECURE_ACCEPT_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_3DSECURE_EXCEPTION_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_3DSECURE_DECLINE_URL)).thenReturn(URL);
    }
}
