package com.psp.payment.directlink.services.impl;

import com.psp.constants.PspConstants;
import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.exception.RequiredFieldException;
import com.psp.payment.directlink.request.PspDirectLinkRequest;
import com.psp.payment.integration.clients.PspRestClient;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import com.psp.payment.integration.utils.PspIntegrationUtils;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.order.CommerceCheckoutService;
import de.hybris.platform.commerceservices.strategies.GenerateMerchantTransactionCodeStrategy;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectLinkServiceImplTest {

    private static final String PSP_ID = "pspid";
    private static final String FRAUD_SCORE_RED = "R";
    private static final String FRAUD_SCORE_ORANGE = "O";
    private static final String FRAUD_SCORE_GREEN = "G";
    private static final String STATUS_PAYMENT_REQUESTED = "9";
    private static final String STATUS_IDENTIFICATION_REQUESTED = "46";

    @InjectMocks
    private PspDirectLinkServiceImpl pspDirectLinkService;

    @Mock
    private PspUtils pspUtils;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private PspRestClient<String> pspRestClient;

    @Mock
    private PspIntegrationUtils pspIntegrationUtils;

    @Mock
    private ModelService modelService;

    @Mock
    private CartService cartService;

    @Mock
    private GenerateMerchantTransactionCodeStrategy generateMerchantTransactionCodeStrategy;

    @Mock
    private CommonI18NService commonI18NService;

    @Mock
    private CommerceCheckoutService commerceCheckoutService;

    @Mock
    private CartModel cartModel;

    @Mock
    private CurrencyModel currencyModel;

    @Mock
    private PaymentService paymentService;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    private PspDirectLinkRequest pspDirectLinkRequest;

    private PspDirectLinkRequest pspDirectLink3DSecureRequest;

    private PaymentTransactionModel paymentTransactionModel;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private Map<String, String> parameters;

    @Before
    public void setup() {
        pspDirectLinkRequest = new PspDirectLinkRequest();
        pspDirectLink3DSecureRequest = new PspDirectLinkRequest();
        createDirectLinkRequest(pspDirectLinkRequest);
        createDirectLink3DSecureRequest(pspDirectLink3DSecureRequest);
        setupMockingBehaviour();
        parameters = getInputParameters("20.00", "payid", FRAUD_SCORE_GREEN, STATUS_PAYMENT_REQUESTED);
    }

    @Test(expected = RequiredFieldException.class)
    public void lessThanRequiredAttributesProvided() {
        // Given less than mandatory attributes

        // When
        pspDirectLinkService.executeDirectLinkRequest(pspDirectLinkRequest);

        // Then an exception is thrown
    }

    @Test
    public void allRequiredAttributesProvided() {
        // Given complete mandatory attributes
        pspDirectLinkRequest.setPspId(PSP_ID);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PASSPHRASE)).thenReturn("passphrase");
        when(pspRestClient.postRequest(any(URI.class), anyMap(), eq(String.class))).thenReturn("response");

        // When
        pspDirectLinkService.executeDirectLinkRequest(pspDirectLinkRequest);

        // Then
        verify(pspUtils).generateChecksum(anyMap(), eq("passphrase"));
        verify(pspRestClient).postRequest(any(URI.class), anyMap(), eq(String.class));
        verify(pspIntegrationUtils).convertPlainStringToParameters("response");
    }

    @Test(expected = RequiredFieldException.class)
    public void lessThanRequiredAttributesProvidedFor3DSecure() {
        // Given less than mandatory attributes

        // When
        pspDirectLinkService.executeDirectLinkRequest(pspDirectLink3DSecureRequest);

        // Then an exception is thrown
    }

    @Test
    public void allRequiredAttributesProvidedFor3DSecure() {
        // Given complete mandatory attributes
        pspDirectLink3DSecureRequest.setPspId(PSP_ID);
        when(pspUtils.is3DSecureEnabled()).thenReturn(Boolean.TRUE);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PASSPHRASE)).thenReturn("passphrase");
        when(pspRestClient.postRequest(any(URI.class), anyMap(), eq(String.class))).thenReturn("response");

        // When
        pspDirectLinkService.executeDirectLinkRequest(pspDirectLink3DSecureRequest);

        // Then
        verify(pspUtils).generateChecksum(anyMap(), eq("passphrase"));
        verify(pspRestClient).postRequest(any(URI.class), anyMap(), eq(String.class));
        verify(pspIntegrationUtils).convertPlainStringToParameters("response");
    }

    @Test(expected = IllegalArgumentException.class)
    public void testExecuteDirectLinkRequestNull() {
        // When
        pspDirectLinkService.executeDirectLinkRequest(null);

        // Then an exception will be thrown
    }

    @Test
    public void createSuccessfulPaymentTransactionEntry() {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(PaymentTransactionType.CAPTURE);

        // When
        PaymentTransactionEntryModel result = pspDirectLinkService.createPaymentTransactionAndEntry(parameters);

        // Then
        assertNotNull(result);
        assertNotNull(result.getPaymentTransaction());
        assertEquals(PaymentTransactionType.CAPTURE, result.getType());
        assertEquals(new BigDecimal("20.00"), result.getAmount());
        assertEquals("payid", result.getRequestToken());
        assertEquals(TransactionStatus.ACCEPTED.name(), result.getTransactionStatus());
        assertEquals(TransactionStatusDetails.SUCCESFULL.name(), result.getTransactionStatusDetails());
        verify(modelService).saveAll(paymentTransactionEntryModel, paymentTransactionModel);
    }

    @Test
    public void createReviewPaymentTransactionEntry() {
        // Given
        parameters.put(PspConstants.KEY_RESULT_FRAUD_CATEGORY, FRAUD_SCORE_ORANGE);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(PaymentTransactionType.CAPTURE);

        // When
        PaymentTransactionEntryModel result = pspDirectLinkService.createPaymentTransactionAndEntry(parameters);

        // Then
        assertNotNull(result);
        assertNotNull(result.getPaymentTransaction());
        assertEquals(PaymentTransactionType.CAPTURE, result.getType());
        assertEquals(new BigDecimal("20.00"), result.getAmount());
        assertEquals("payid", result.getRequestToken());
        assertEquals(TransactionStatus.REVIEW.name(), result.getTransactionStatus());
        assertEquals(TransactionStatusDetails.SUCCESFULL.name(), result.getTransactionStatusDetails());
        verify(modelService).saveAll(paymentTransactionEntryModel, paymentTransactionModel);
    }

    @Test
    public void createSuccessfulPaymentTransactionEntryWithoutFraudScore() {
        // Given
        parameters.remove(PspConstants.KEY_RESULT_FRAUD_CATEGORY);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(PaymentTransactionType.CAPTURE);

        // When
        PaymentTransactionEntryModel result = pspDirectLinkService.createPaymentTransactionAndEntry(parameters);

        // Then
        assertNotNull(result);
        assertNotNull(result.getPaymentTransaction());
        assertEquals(PaymentTransactionType.CAPTURE, result.getType());
        assertEquals(new BigDecimal("20.00"), result.getAmount());
        assertEquals("payid", result.getRequestToken());
        assertEquals(TransactionStatus.ACCEPTED.name(), result.getTransactionStatus());
        assertEquals(TransactionStatusDetails.SUCCESFULL.name(), result.getTransactionStatusDetails());
        verify(modelService).saveAll(paymentTransactionEntryModel, paymentTransactionModel);
    }

    @Test
    public void getTransactionStatusAsAccepted() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_PAYMENT_REQUESTED);

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.ACCEPTED.name(), result);
    }

    @Test
    public void getTransactionStatusAsAcceptedForFraudCheck() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_PAYMENT_REQUESTED);
        parameters.put(PspConstants.KEY_RESULT_FRAUD_CATEGORY, FRAUD_SCORE_GREEN);

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.ACCEPTED.name(), result);
    }

    @Test
    public void getTransactionStatusAsReviewForFraudCheck() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_PAYMENT_REQUESTED);
        parameters.put(PspConstants.KEY_RESULT_FRAUD_CATEGORY, FRAUD_SCORE_ORANGE);

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.REVIEW.name(), result);
    }

    @Test
    public void getTransactionStatusAsRejectedForFraudCheck() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_PAYMENT_REQUESTED);
        parameters.put(PspConstants.KEY_RESULT_FRAUD_CATEGORY, FRAUD_SCORE_RED);

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.REJECTED.name(), result);
    }

    @Test
    public void getTransactionStatusAsAcceptedFor3DSecure() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_IDENTIFICATION_REQUESTED);

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.ACCEPTED.name(), result);
    }

    @Test
    public void getTransactionStatusAsRejected() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, "0");

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.REJECTED.name(), result);
    }

    @Test
    public void getSuccessfulTransactionStatusDetails() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_PAYMENT_REQUESTED);

        // When
        TransactionStatusDetails result = pspDirectLinkService.getTransactionStatusDetails(parameters);

        // Then
        assertEquals(TransactionStatusDetails.SUCCESFULL, result);
    }

    @Test
    public void getAuthenticationRequiredTransactionStatusDetails() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, STATUS_IDENTIFICATION_REQUESTED);

        // When
        TransactionStatusDetails result = pspDirectLinkService.getTransactionStatusDetails(parameters);

        // Then
        assertEquals(TransactionStatusDetails.THREE_D_SECURE_AUTHENTICATION_REQUIRED, result);
    }

    @Test
    public void getRejectedTransactionStatusDetails() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, "0");

        // When
        TransactionStatusDetails result = pspDirectLinkService.getTransactionStatusDetails(parameters);

        // Then
        assertEquals(TransactionStatusDetails.AUTHORIZATION_REJECTED_BY_PSP, result);
    }

    @Test
    public void getTransactionStatusAsUncertain() {
        // Given
        Map<String, String> testParameters = new HashMap<>();
        testParameters.put(PspConstants.KEY_STATUS, String.valueOf(PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN));
        when(configurationService.getConfiguration().getBoolean(PspConstants.PARAM_UNCERTAIN_STATUS_HANDLING_ENABELD)).thenReturn(Boolean.TRUE);

        // When
        String result = pspDirectLinkService.getTransactionStatus(testParameters);

        // Then
        assertEquals(PspTransactionStatus.UNCERTAIN.name(), result);
    }

    @Test
    public void getReviewTransactionStatusDetails() {
        // Given
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, String.valueOf(PspPaymentStatus.STATUS_AUTHORISATION_NOT_KNOWN));

        // When
        TransactionStatusDetails result = pspDirectLinkService.getTransactionStatusDetails(parameters);

        // Then
        assertEquals(TransactionStatusDetails.SUCCESFULL, result);
    }

    @Test
    public void getAcceptedStatusWithUncertainParamDisabled() {
        // Given
        when(configurationService.getConfiguration().getBoolean(anyString())).thenReturn(Boolean.FALSE);
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_STATUS, String.valueOf(PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN));

        // When
        String result = pspDirectLinkService.getTransactionStatus(parameters);

        // Then
        assertEquals(TransactionStatus.ACCEPTED.name(), result);
    }

    private void createDirectLinkRequest(PspDirectLinkRequest pspDirectLinkRequest) {
        pspDirectLinkRequest.setUserId("userid");
        pspDirectLinkRequest.setPassword("password");
        pspDirectLinkRequest.setOrderId("orderid");
        pspDirectLinkRequest.setCurrency("currency");
        pspDirectLinkRequest.setAmount(0);
        pspDirectLinkRequest.setOperation("operation");
        pspDirectLinkRequest.setEci("eci");
        pspDirectLinkRequest.setAlias("alias");
    }

    private void createDirectLink3DSecureRequest(PspDirectLinkRequest pspDirectLinkRequest) {
        createDirectLinkRequest(pspDirectLinkRequest);
        pspDirectLinkRequest.setFlag3d("flag3d");
        pspDirectLinkRequest.setHttpAccept("httpaccept");
        pspDirectLinkRequest.setWin3ds("win3ds");
        pspDirectLinkRequest.setAcceptUrl("accepturl");
        pspDirectLinkRequest.setDeclineUrl("declineurl");
        pspDirectLinkRequest.setExceptionUrl("exceptionurl");
        pspDirectLinkRequest.setLanguage("language");
    }

    private Map<String, String> getInputParameters(String amount, String payId, String fraudScore, String transactionStatus) {
        Map<String, String> parameters = new HashMap<>();
        parameters.put(PspConstants.KEY_CURRENCY, "GBP");
        parameters.put(PspConstants.KEY_PAYID, payId);
        parameters.put(PspConstants.KEY_RESULT_FRAUD_CATEGORY, fraudScore);
        parameters.put(PspConstants.KEY_RESULT_AMOUNT, amount);
        parameters.put(PspConstants.KEY_STATUS, transactionStatus);
        return parameters;
    }

    private void setupMockingBehaviour() {
        CreditCardPaymentInfoModel paymentInfoModel = new CreditCardPaymentInfoModel();
        CartModel cartModel = new CartModel();
        cartModel.setPaymentInfo(paymentInfoModel);
        paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        CurrencyModel currencyModel = new CurrencyModel();

        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(modelService.create(PaymentTransactionModel.class)).thenReturn(paymentTransactionModel);
        when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(paymentTransactionEntryModel);
        when(generateMerchantTransactionCodeStrategy.generateCode(cartModel)).thenReturn("testpt");
        when(commonI18NService.getCurrency("GBP")).thenReturn(currencyModel);
        when(commerceCheckoutService.getPaymentProvider()).thenReturn("test");
    }
}
