package com.psp.payment.flexpayments.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.services.PspDynamicPaymentTemplateService;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.strategies.PspAddressBuilderStrategy;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.commerceservices.i18n.CommerceCommonI18NService;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.commerceservices.strategies.CustomerNameStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.site.BaseSiteService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Optional;

import static com.psp.constants.PspConstants.*;
import static java.lang.Boolean.TRUE;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.mockito.Matchers.anyMap;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCreateFlexPaymentRequestStrategyImplTest {

    private static final String UNIQUE_ORDER_ID = "100000";
    private static final String SHA_SIGN = "sha";
    private static final String URL = "url";
    private static final String PSP_ID = "pspid";
    private static final String LANGUAGE_CODE = "EN";
    private static final String CURRENCY_CODE = "GBP";
    private static final String PAYMENT_METHOD = "PayPal";
    private static final String BRAND = "PayPal";
    private static final String EMAIL = "test@user.com";

    @InjectMocks
    private PspCreateFlexPaymentRequestStrategyImpl pspCreateFlexPaymentRequestStrategy;

    @Mock
    private PspUtils pspUtils;

    @Mock
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private CommerceCommonI18NService commerceCommonI18NService;

    @Mock
    private CustomerModel customerModel;

    @Mock
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Mock
    private PspDynamicPaymentTemplateService pspDynamicPaymentTemplateService;

    @Mock
    private CartModel cartModel;

    @Mock
    private AddressModel addressModel;

    @Mock
    private CustomerNameStrategy customerNameStrategy;

    @Mock
    private PspAddressBuilderStrategy pspAddressBuilderStrategy;

    @Mock
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private BaseSiteModel baseSiteModel;

    @Before
    public void setUp() {
        setupMockingBehaviour();
    }

    @Test
    public void createSubscriptionRequestIsValid() {
        // Given
        PspFlexPaymentRequest expectedResult = createRequest();

        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel);

        // Then
        assertEquals(expectedResult, result);
    }

    @Test
    public void createSubscriptionRequestWithInvalidParameters() {
        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(null);

        // Then
        PspFlexPaymentRequest expectedResult = new PspFlexPaymentRequest();
        assertEquals(expectedResult, result);
    }

    @Test
    public void emailIsPopulatedFromCustomerIfNotProvidedInAddress() {
        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel);

        // Then
        assertEquals(EMAIL, result.getEmail());
        verify(checkoutCustomerStrategy).getCurrentUserForCheckout();
        verify(customerEmailResolutionService).getEmailForCustomer(customerModel);
    }

    @Test
    public void emailIsPopulatedFromAddressWhenPresent() {
        // Given
        when(addressModel.getEmail()).thenReturn("fake-email");

        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel);

        // Then
        assertEquals("fake-email", result.getEmail());
        verify(checkoutCustomerStrategy, never()).getCurrentUserForCheckout();
        verify(customerEmailResolutionService, never()).getEmailForCustomer(customerModel);
    }

    @Test
    public void trackingKeyIsPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn("HYB010317");

        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel);

        // Then
        assertEquals("HYB010317", result.getTrackingKey());
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn(null);

        // When
        PspFlexPaymentRequest result = pspCreateFlexPaymentRequestStrategy.createFlexPaymentRequest(cartModel);

        // Then
        assertNull(result.getTrackingKey());
    }

    private PspFlexPaymentRequest createRequest() {
        PspFlexPaymentRequest expectedResult = new PspFlexPaymentRequest();
        expectedResult.setOrderId(UNIQUE_ORDER_ID);
        expectedResult.setExceptionUrl(URL);
        expectedResult.setAcceptUrl(URL);
        expectedResult.setDeclineUrl(URL);
        expectedResult.setCancelUrl(URL);
        expectedResult.setPaymentMethod(PAYMENT_METHOD);
        expectedResult.setBrand(BRAND);
        expectedResult.setPspId(PSP_ID);
        expectedResult.setLanguage(LANGUAGE_CODE);
        expectedResult.setCurrency(CURRENCY_CODE);
        expectedResult.setShaSign(SHA_SIGN);
        expectedResult.setAmount(BigDecimal.ZERO);
        expectedResult.setEmail(EMAIL);
        expectedResult.setBackUrl(URL);
        expectedResult.setTemplate("url");
        expectedResult.setCustomerName("first last");
        expectedResult.setOwnerAddress("street number");
        return expectedResult;
    }

    private void setupMockingBehaviour() {
        PaymentModeModel paymentModeModel = mock(PaymentModeModel.class);
        when(paymentModeModel.getBrand()).thenReturn("PayPal");
        when(paymentModeModel.getPaymentMethod()).thenReturn("PayPal");
        when(addressModel.getFirstname()).thenReturn("first");
        when(addressModel.getLastname()).thenReturn("last");
        when(addressModel.getStreetname()).thenReturn("street");
        when(addressModel.getStreetnumber()).thenReturn("number");
        when(cartModel.getPaymentMode()).thenReturn(paymentModeModel);
        when(cartModel.getDeliveryAddress()).thenReturn(addressModel);
        when(cartModel.getNet()).thenReturn(Boolean.FALSE);

        when(pspUtils.generateUniqueOrderId(cartModel)).thenReturn(UNIQUE_ORDER_ID);
        when(pspUtils.generateChecksum(anyMap(), anyString())).thenReturn(SHA_SIGN);
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_FLEX_EXCEPTION_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_FLEX_ACCEPT_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_FLEX_DECLINE_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_FLEX_CANCEL_URL)).thenReturn(URL);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_FLEX_BACK_URL)).thenReturn(URL);
        when(configurationService.getConfiguration().getString(PARAM_FLEX_EXCEPTION_URL)).thenReturn(PARAM_FLEX_EXCEPTION_URL);
        when(configurationService.getConfiguration().getString(PARAM_FLEX_ACCEPT_URL)).thenReturn(PARAM_FLEX_ACCEPT_URL);
        when(configurationService.getConfiguration().getString(PARAM_FLEX_DECLINE_URL)).thenReturn(PARAM_FLEX_DECLINE_URL);
        when(configurationService.getConfiguration().getString(PARAM_FLEX_CANCEL_URL)).thenReturn(PARAM_FLEX_CANCEL_URL);
        when(configurationService.getConfiguration().getString(PARAM_FLEX_BACK_URL)).thenReturn(PARAM_FLEX_BACK_URL);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PSPID)).thenReturn(PSP_ID);
        when(commerceCommonI18NService.getCurrentLanguage().getIsocode()).thenReturn(LANGUAGE_CODE);
        when(commerceCommonI18NService.getCurrentCurrency().getIsocode()).thenReturn(CURRENCY_CODE);
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(customerModel);
        when(customerEmailResolutionService.getEmailForCustomer(customerModel)).thenReturn(EMAIL);
        when(pspUtils.getWebsiteIdByCart(cartModel)).thenReturn("apparel-uk");
        when(pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl()).thenReturn(Optional.of("url"));
        when(customerNameStrategy.getName("first", "last")).thenReturn("first last");
        when(pspAddressBuilderStrategy.buildAddress("street", "number")).thenReturn("street number");
    }
}