package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.constants.PspConstants;
import com.psp.payment.fraud.PspFraudService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.enums.FraudStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.fraud.impl.FraudSymptom;
import de.hybris.platform.fraud.model.FraudReportModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.configuration.Configuration;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFraudCheckOrderActionTest {

    @InjectMocks
    private PspFraudCheckOrderAction pspFraudCheckOrderAction;

    @Mock
    private ModelService modelService;

    @Mock
    private PspFraudService fraudService;

    @Mock
    private ConfigurationService configurationService;

    @Mock
    private Configuration configuration;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private FraudReportModel fraudReport;

    private static final String PROVIDER_NAME = "psp";

    private OrderProcessModel process;

    private FraudServiceResponse response;

    private FraudSymptom symptom;

    @Before
    public void setUp() {
        pspFraudCheckOrderAction.setProviderName(PROVIDER_NAME);
        response = new FraudServiceResponse(PROVIDER_NAME);

        setupMockingBehaviour();
    }

    @Test
    public void executePotentialFraud() throws Exception {
        // Given
        process = createOrderProcess(TransactionStatusDetails.REVIEW_NEEDED.name());
        symptom = new FraudSymptom("PspFraudScoreSymptom", 50);
        response.addSymptom(symptom);

        // When
        String result = pspFraudCheckOrderAction.execute(process);

        //Then
        assertTrue(PspFraudCheckOrderAction.Transition.POTENTIAL.name().equals(result));
    }

    @Test
    public void executeFraudOK() throws Exception {
        // Given
        process = createOrderProcess(TransactionStatusDetails.SUCCESFULL.name());
        symptom = new FraudSymptom("PspFraudScoreSymptom", 10);
        response.addSymptom(symptom);

        // When
        String result = pspFraudCheckOrderAction.execute(process);

        //Then
        assertTrue(PspFraudCheckOrderAction.Transition.OK.name().equals(result));
    }

    @Test
    public void executeFraud() throws Exception {
        // Given
        process = createOrderProcess(TransactionStatusDetails.SUCCESFULL.name());
        symptom = new FraudSymptom("PspFraudScoreSymptom", 100);
        response.addSymptom(symptom);

        // When
        String result = pspFraudCheckOrderAction.execute(process);

        //Then
        assertTrue(PspFraudCheckOrderAction.Transition.FRAUD.name().equals(result));
    }


    @Test
    public void expectedTransitionsAreThere() {
        // Given
        Set<String> expectedTransitions = new HashSet<>();
        expectedTransitions.add("OK");
        expectedTransitions.add("FRAUD");
        expectedTransitions.add("POTENTIAL");

        // When
        Set<String> transitions = pspFraudCheckOrderAction.getTransitions();

        // Then
        assertEquals(expectedTransitions, transitions);
    }

    private OrderProcessModel createOrderProcess(String status) {
        OrderModel order = new OrderModel();
        PaymentTransactionModel paymentTransactionModel = getTransaction(status);
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode("123");
        paymentTransactionModel.setOrder(order);
        OrderProcessModel process = new OrderProcessModel();
        process.setOrder(order);
        return process;
    }

    private PaymentTransactionModel getTransaction(String status) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionEntryModel.setTransactionStatusDetails(status);
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));
        return paymentTransactionModel;
    }

    private void setupMockingBehaviour() {
        when(configurationService.getConfiguration()).thenReturn(configuration);
        when(configuration.getDouble(PspConstants.PARAM_FRAUD_SCORE_LIMIT)).thenReturn(50.0d);
        when(configuration.getDouble(PspConstants.PARAM_FRAUD_SCORE_TOLERANCE)).thenReturn(10.0d);
        when(fraudService.recognizeOrderSymptoms(anyString(), any(OrderModel.class))).thenReturn(response);
        when(fraudService.createFraudReport(anyString(), any(FraudServiceResponse.class), any(OrderModel.class), any(FraudStatus.class))).thenReturn(fraudReport);
    }
}