package com.psp.payment.maintenance.services.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.integration.clients.PspRestClient;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import com.psp.payment.integration.utils.PspIntegrationUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspMaintenanceRequestCreationStrategy;
import com.psp.payment.maintenance.strategies.impl.PspMaintenanceResponseParsingStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenanceServiceImplTest {

    @InjectMocks
    private PspMaintenanceServiceImpl pspMaintenanceService;

    @Mock
    private PspMaintenanceRequestCreationStrategy pspMaintenanceRequestCreationStrategy;

    @Mock
    private PspMaintenanceRequest pspMaintenanceRequest;

    @Mock
    private Converter<PspMaintenanceRequest, Map<String, String>> pspMaintenanceRequestConverter;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private PspUtils pspUtils;

    @Mock
    private PspRestClient<String> pspRestClient;

    @Mock
    private PspIntegrationUtils pspIntegrationUtils;

    @Mock
    private Converter<Map<String, String>, PspMaintenanceResponse> pspMaintenanceResponseConverter;

    @Mock
    private PspMaintenanceResponseParsingStrategyImpl pspMaintenanceResponseParsingStrategy;

    @Mock
    private OrderModel orderModel;

    @Mock
    private BigDecimal proposedAmount;

    @Before
    public void setup() {
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_MAINTENANCE_URL)).thenReturn("http://www.google.com");
    }

    @Test(expected = IllegalArgumentException.class)
    public void captureFailsWhenOrderNotProvided() throws URISyntaxException {
        // When
        pspMaintenanceService.captureFunds(null, proposedAmount);

        // Then an exception will be thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void captureFailsWhenProposedAmountNotProvided() throws URISyntaxException {
        // When
        pspMaintenanceService.captureFunds(orderModel, null);

        // Then an exception will be thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureFailsWhenRequestNotCreated() throws URISyntaxException {
        // Given
        when(pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount)).thenThrow(IllegalMaintenanceOperation.class);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then an exception will be thrown
    }

    @Test
    public void captureRequestIsCreated() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspMaintenanceRequestCreationStrategy).createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);
    }

    @Test
    public void refundRequestIsCreated() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.refundFunds(orderModel, proposedAmount);

        // Then
        verify(pspMaintenanceRequestCreationStrategy).createMaintenanceRequest(PspMaintenanceRequestType.REFUND, orderModel, proposedAmount);
    }

    @Test
    public void requestParameterMapIsPopulated() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount)).thenReturn(pspMaintenanceRequest);
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspMaintenanceRequestConverter).convert(eq(pspMaintenanceRequest), anyMap());
    }

    @Test
    public void checksumIsGenerated() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PASSPHRASE)).thenReturn("passphrase");
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspUtils).generateChecksum(anyMap(), eq("passphrase"));
    }

    @Test
    public void restCallIsMadeToPsp() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspRestClient).postRequest(any(URI.class), anyMap(), eq(String.class));
    }

    @Test(expected = URISyntaxException.class)
    public void invalidUrlFailsMaintenanceCall() throws URISyntaxException {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_MAINTENANCE_URL)).thenReturn("[^&$^&]");

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then an exception is thrown
    }

    @Test
    public void responseStringIsConvertedToParameters() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));
        when(pspRestClient.postRequest(any(URI.class), anyMap(), eq(String.class))).thenReturn("response");
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspIntegrationUtils).convertPlainStringToParameters("response");
    }

    @Test
    public void responseDtoIsPopulated() throws URISyntaxException {
        // Given
        Map<String, String> parameterMap = new HashMap<>();
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        when(pspIntegrationUtils.convertPlainStringToParameters(anyString())).thenReturn(parameterMap);
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspMaintenanceResponseConverter).convert(eq(parameterMap), any(PspMaintenanceResponse.class));
    }

    @Test
    public void responseParsingStrategyIsCalled() throws URISyntaxException {
        // Given
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        when(pspMaintenanceResponseConverter.convert(anyMap(), any(PspMaintenanceResponse.class))).thenReturn(pspMaintenanceResponse);

        // When
        pspMaintenanceService.captureFunds(orderModel, proposedAmount);

        // Then
        verify(pspMaintenanceResponseParsingStrategy).parseResponse(eq(PspMaintenanceRequestType.CAPTURE), eq(orderModel), eq(pspMaintenanceResponse));
    }
}
