package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.paymentstandard.model.StandardPaymentModeModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static de.hybris.platform.payment.enums.PaymentTransactionType.AUTHORIZATION;
import static de.hybris.platform.payment.enums.PaymentTransactionType.CAPTURE;
import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCaptureEvaluationStrategyImplTest {

    @InjectMocks
    private PspCaptureEvaluationStrategyImpl pspCaptureOperationStrategy;

    private OrderModel orderModel;

    private List<PaymentTransactionModel> paymentTransactionModelList;

    private StandardPaymentModeModel standardPaymentModeModel;

    @Before
    public void setup() {
        orderModel = new OrderModel();
        standardPaymentModeModel = new StandardPaymentModeModel();
        paymentTransactionModelList = new ArrayList<>();
        setupPaymentTransaction(AUTHORIZATION, new BigDecimal("30.0"));
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void partialCaptureNotCalculatedIfUnsupported() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test
    public void partialCaptureOperationIsCalculated() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");
        setPaymentModeForPartialOperation(Boolean.TRUE);

        // When
        PspOperation result = pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(PspOperation.SAL, result);
    }

    @Test
    public void fullCaptureOperationIsCalculated() {
        // Given
        BigDecimal amount = new BigDecimal("30.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        PspOperation result = pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(PspOperation.SAS, result);
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForAmountGreaterThanAuthorisedAmount() {
        // Given
        BigDecimal amount = new BigDecimal("40.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void captureNotCalculatedWhenOrderNotProvided() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(null, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void captureNotCalculatedWhenAmountNotProvided() {
        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, null);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForNegativeAmount() {
        // Given
        BigDecimal amount = new BigDecimal("-10.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForAlreadyFullyCapturedOrder() {
        // Given all £30 has already been captured
        setupPaymentTransaction(CAPTURE, new BigDecimal("30.0"));
        BigDecimal amount = new BigDecimal("1.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForTotalAmountGreaterThanAuthorisedOrder() {
        // Given £20 has already been captured of £30 authorisation
        setupPaymentTransaction(CAPTURE, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test
    public void partialCaptureCalculatedForTotalAmountLessThanAuthorisedOrder() {
        // Given £20 has already been captured of £30 authorisation
        setupPaymentTransaction(CAPTURE, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("5.0");
        setPaymentModeForPartialOperation(Boolean.TRUE);

        // When
        PspOperation result = pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(PspOperation.SAL, result);
    }

    @Test
    public void fullCaptureCalculatedForTotalAmountEqualToAuthorisedOrder() {
        // Given £20 has already been captured of £30 authorisation
        setupPaymentTransaction(CAPTURE, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("10.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        PspOperation result = pspCaptureOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(PspOperation.SAS, result);
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForOrderWithNoAuthorisations() {
        OrderModel orderWithoutAuthorisation = setupOrderWithSpecificDetails(CAPTURE, new BigDecimal("30.0"));
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderWithoutAuthorisation, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForOrderWithoutTransactions() {
        OrderModel orderWithoutAuthorisation = new OrderModel();
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderWithoutAuthorisation, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void captureNotCalculatedForOrderWithZeroValueAuthorisations() {
        OrderModel orderWithZeroAuthorisation = setupOrderWithSpecificDetails(AUTHORIZATION, BigDecimal.ZERO);
        BigDecimal amount = new BigDecimal("10.0");

        // When
        pspCaptureOperationStrategy.evaluateOperation(orderWithZeroAuthorisation, amount);

        // Then an exception is thrown
    }

    /**
     * Sets up the payment transaction for the given inputs.
     *
     * @param paymentTransactionType the type of payment transaction
     * @param amount                 the amount for payment transaction
     */
    private void setupPaymentTransaction(PaymentTransactionType paymentTransactionType, BigDecimal amount) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(paymentTransactionType);
        paymentTransactionEntryModel.setAmount(amount);
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);
    }

    /**
     * Sets up the payment transaction on a new order for the given inputs.
     *
     * @param paymentTransactionType the type of payment transaction
     * @param amount                 the amount for payment transaction
     * @return the newly created order model
     */
    private OrderModel setupOrderWithSpecificDetails(PaymentTransactionType paymentTransactionType, BigDecimal amount) {
        OrderModel orderModel = new OrderModel();
        List<PaymentTransactionModel> paymentTransactionModelList = new ArrayList<>();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setAmount(amount);
        paymentTransactionEntryModel.setType(paymentTransactionType);
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);

        return orderModel;
    }

    private void setPaymentModeForPartialOperation(Boolean isAccepted) {
        standardPaymentModeModel.setPartialMaintenanceAllowed(isAccepted);
        orderModel.setPaymentMode(standardPaymentModeModel);
    }
}
