package com.psp.payment.maintenance.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenanceRequestCreationStrategyImplTest {

    @InjectMocks
    private PspMaintenanceRequestCreationStrategyImpl pspMaintenanceRequestCreationStrategy;

    @Mock
    private PspCaptureEvaluationStrategyImpl pspCaptureEvaluationStrategy;

    @Mock
    private PspRefundEvaluationStrategyImpl pspRefundEvaluationStrategy;

    @Mock
    private PspUtils pspUtils;

    private OrderModel orderModel;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    private BigDecimal proposedAmount;

    @Before
    public void setup() {
        proposedAmount = new BigDecimal("30.0");
        orderModel = new OrderModel();
        setupPaymentTransactions();
    }

    @Test(expected = IllegalArgumentException.class)
    public void requestNotPopulatedWhenTypeNotProvided() {
        // When
        pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(null, orderModel, proposedAmount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void requestNotPopulatedWhenOrderNotProvided() {
        // When
        pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, null, proposedAmount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void requestNotPopulatedWhenAmountNotProvided() {
        // When
        pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, null);

        // Then an exception is thrown
    }

    @Test
    public void pspIdIsPopulated() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PSPID)).thenReturn("pspid");

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("pspid", result.getPspId());
    }

    @Test
    public void userIdIsPopulated() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_API_USER_ID)).thenReturn("userid");

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("userid", result.getUserId());
    }

    @Test
    public void passwordIsPopulated() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_API_USER_PASSWORD)).thenReturn("password");

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("password", result.getPassword());
    }

    @Test
    public void amountIsPopulated() {
        // Given
        when(pspUtils.getAmountMultipliedBy100(proposedAmount)).thenReturn("3000");

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("3000", result.getAmount());
    }

    @Test(expected = IllegalArgumentException.class)
    public void requestNotCreatedForIncorrectAmount() {
        // Given
        when(pspUtils.getAmountMultipliedBy100(proposedAmount)).thenThrow(IllegalArgumentException.class);

        // When
        pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then an exception will be thrown
    }

    @Test
    public void captureOperationIsPopulated() {
        // Given
        when(pspCaptureEvaluationStrategy.evaluateOperation(orderModel, proposedAmount)).thenReturn(PspOperation.SAS);

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals(PspOperation.SAS, result.getOperation());
    }

    @Test
    public void refundOperationIsPopulated() {
        // Given
        OrderModel orderModel = setupCapturePaymentTransactions();
        when(pspRefundEvaluationStrategy.evaluateOperation(orderModel, proposedAmount)).thenReturn(PspOperation.RFS);

        // When
        PspMaintenanceRequest result = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.REFUND, orderModel, proposedAmount);

        // Then
        assertEquals(PspOperation.RFS, result.getOperation());
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void requestNotCreatedWhenOperationNotEvaluated() {
        // Given
        when(pspCaptureEvaluationStrategy.evaluateOperation(orderModel, proposedAmount)).thenThrow(IllegalMaintenanceOperation.class);

        // When
        pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then an exception will be thrown
    }

    @Test
    public void payIdIsPopulated() {
        // When
        PspMaintenanceRequest pspMaintenanceRequest = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("token", pspMaintenanceRequest.getPayId());
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void requestNotCreatedWhenPayIdNotFound() {
        // Given
        orderModel.setPaymentTransactions(null);

        // When
        PspMaintenanceRequest pspMaintenanceRequest = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);

        // Then
        assertEquals("token", pspMaintenanceRequest.getPayId());
    }

    private void setupPaymentTransactions() {
        List<PaymentTransactionModel> paymentTransactionModelList = new ArrayList<>();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(PaymentTransactionType.AUTHORIZATION);
        paymentTransactionEntryModel.setRequestToken("token");
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);
    }

    private OrderModel setupCapturePaymentTransactions() {
        OrderModel orderModel = new OrderModel();
        List<PaymentTransactionModel> paymentTransactionModelList = new ArrayList<>();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(PaymentTransactionType.CAPTURE);
        paymentTransactionEntryModel.setRequestToken("token");
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);

        return orderModel;
    }

}
