package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspMaintenancePaymentTransactionCreationStrategy;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Mockito.verify;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenanceResponseParsingStrategyImplTest {

    @InjectMocks
    PspMaintenanceResponseParsingStrategyImpl pspMaintenanceResponseParsingStrategy;

    @Mock
    private PspMaintenancePaymentTransactionCreationStrategy pspMaintenancePaymentTransactionCreationStrategy;

    private OrderModel orderModel;

    private PspMaintenanceResponse pspMaintenanceResponse;

    @Before
    public void setup() {
        orderModel = new OrderModel();
        pspMaintenanceResponse = new PspMaintenanceResponse();
    }

    @Test(expected = IllegalArgumentException.class)
    public void parsingFailsWhenTransactionTypeNotProvided() {
        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(null, orderModel, pspMaintenanceResponse);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void parsingFailsWhenOrderNotProvided() {
        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.CAPTURE, null, pspMaintenanceResponse);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void parsingFailsWhenResponseNotProvided() {
        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.CAPTURE, orderModel, null);

        // Then an exception is thrown
    }

    @Test
    public void captureProcessingCreatesPaymentTransaction() {
        // Given
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.CAPTURE, orderModel, pspMaintenanceResponse);

        // Then
        verify(pspMaintenancePaymentTransactionCreationStrategy).createPaymentTransaction(PaymentTransactionType.CAPTURE, orderModel, pspMaintenanceResponse);
    }

    @Test
    public void captureRequestedCreatesPaymentTransaction() {
        // Given
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_REQUESTED));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.CAPTURE, orderModel, pspMaintenanceResponse);

        // Then
        verify(pspMaintenancePaymentTransactionCreationStrategy).createPaymentTransaction(PaymentTransactionType.CAPTURE, orderModel, pspMaintenanceResponse);
    }

    @Test
    public void refundPendingCreatesPaymentTransaction() {
        // Given
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_REFUND_PENDING));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.REFUND, orderModel, pspMaintenanceResponse);

        // Then
        verify(pspMaintenancePaymentTransactionCreationStrategy).createPaymentTransaction(PaymentTransactionType.REFUND_FOLLOW_ON, orderModel, pspMaintenanceResponse);
    }

    @Test
    public void refundCreatesPaymentTransaction() {
        // Given
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_REFUND));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.REFUND, orderModel, pspMaintenanceResponse);

        // Then
        verify(pspMaintenancePaymentTransactionCreationStrategy).createPaymentTransaction(PaymentTransactionType.REFUND_FOLLOW_ON, orderModel, pspMaintenanceResponse);
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void failsWhenCaptureStatusIsAnythingElse() {
        // Given that status is any other than processing or requested for capture request
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_REFUSED));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.CAPTURE, orderModel, pspMaintenanceResponse);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void failsWhenRefundStatusIsAnythingElse() {
        // Given that status is any other than refund or refund pending for refund request
        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setStatus(String.valueOf(PspPaymentStatus.STATUS_REFUND_REFUSED));

        // When
        pspMaintenanceResponseParsingStrategy.parseResponse(PspMaintenanceRequestType.REFUND, orderModel, pspMaintenanceResponse);

        // Then an exception is thrown
    }
}
