package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.paymentstandard.model.StandardPaymentModeModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static com.psp.payment.common.enums.PspOperation.RFD;
import static com.psp.payment.common.enums.PspOperation.RFS;
import static de.hybris.platform.payment.enums.PaymentTransactionType.*;
import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspRefundEvaluationStrategyImplTest {

    @InjectMocks
    private PspRefundEvaluationStrategyImpl pspRefundOperationStrategy;

    private OrderModel orderModel;

    private List<PaymentTransactionModel> paymentTransactionModelList;

    private StandardPaymentModeModel standardPaymentModeModel;

    @Before
    public void setup() {
        orderModel = new OrderModel();
        standardPaymentModeModel = new StandardPaymentModeModel();
        paymentTransactionModelList = new ArrayList<>();
        setupPaymentTransaction(CAPTURE, new BigDecimal("30.0"));
    }

    @Test(expected = IllegalArgumentException.class)
    public void refundNotCalculatedWhenOrderNotProvided() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(null, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void refundNotCalculatedWhenAmountNotProvided() {
        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, null);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForNegativeAmount() {
        // Given
        BigDecimal amount = new BigDecimal("-10.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForAmountGreaterThanCapturedAmount() {
        // Given
        BigDecimal amount = new BigDecimal("40.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForAlreadyFullyRefundedOrder() {
        // Given all £30 has already been refunded
        setupPaymentTransaction(REFUND_FOLLOW_ON, new BigDecimal("30.0"));
        BigDecimal amount = new BigDecimal("1.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForTotalAmountGreaterThanCapturedAmount() {
        // Given £20 has already been refunded for £30 capture
        setupPaymentTransaction(REFUND_FOLLOW_ON, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForOrderWithNoCaptures() {
        OrderModel orderWithoutAuthorisation = setupOrderWithSpecificDetails(AUTHORIZATION, new BigDecimal("30.0"));
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderWithoutAuthorisation, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForOrderWithoutTransactions() {
        OrderModel orderWithoutAuthorisation = new OrderModel();
        BigDecimal amount = new BigDecimal("11.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderWithoutAuthorisation, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void refundNotCalculatedForOrderWithZeroValueCaptures() {
        OrderModel orderWithZeroAuthorisation = setupOrderWithSpecificDetails(CAPTURE, BigDecimal.ZERO);
        BigDecimal amount = new BigDecimal("10.0");

        // When
        pspRefundOperationStrategy.evaluateOperation(orderWithZeroAuthorisation, amount);

        // Then an exception is thrown
    }

    @Test(expected = IllegalMaintenanceOperation.class)
    public void partialRefundNotCalculatedIfUnsupported() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then an exception is thrown
    }

    @Test
    public void partialRefundOperationIsCalculated() {
        // Given
        BigDecimal amount = new BigDecimal("20.0");
        setPaymentModeForPartialOperation(Boolean.TRUE);

        // When
        PspOperation result = pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(RFD, result);
    }

    @Test
    public void fullRefundOperationIsCalculated() {
        // Given
        BigDecimal amount = new BigDecimal("30.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        PspOperation result = pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(RFS, result);
    }

    @Test
    public void partialRefundCalculatedForTotalAmountLessThanCaptured() {
        // Given £20 has already been refunded of £30 captured
        setupPaymentTransaction(REFUND_FOLLOW_ON, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("5.0");
        setPaymentModeForPartialOperation(Boolean.TRUE);

        // When
        PspOperation result = pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(RFD, result);
    }

    @Test
    public void fullRefundCalculatedForTotalAmountEqualToCaptured() {
        // Given £20 has already been refunded of £30 captured
        setupPaymentTransaction(REFUND_FOLLOW_ON, new BigDecimal("20.0"));
        BigDecimal amount = new BigDecimal("10.0");
        setPaymentModeForPartialOperation(Boolean.FALSE);

        // When
        PspOperation result = pspRefundOperationStrategy.evaluateOperation(orderModel, amount);

        // Then
        assertEquals(RFS, result);
    }

    /**
     * Sets up the payment transaction for the given inputs.
     *
     * @param paymentTransactionType the type of payment transaction
     * @param amount                 the amount for payment transaction
     */
    private void setupPaymentTransaction(PaymentTransactionType paymentTransactionType, BigDecimal amount) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setType(paymentTransactionType);
        paymentTransactionEntryModel.setAmount(amount);
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);
    }

    /**
     * Sets up the payment transaction on a new order for the given inputs.
     *
     * @param paymentTransactionType the type of payment transaction
     * @param amount                 the amount for payment transaction
     * @return the newly created order model
     */
    private OrderModel setupOrderWithSpecificDetails(PaymentTransactionType paymentTransactionType, BigDecimal amount) {
        OrderModel orderModel = new OrderModel();
        List<PaymentTransactionModel> paymentTransactionModelList = new ArrayList<>();
        List<PaymentTransactionEntryModel> paymentTransactionEntryModelList = new ArrayList<>();
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        PaymentTransactionEntryModel paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setAmount(amount);
        paymentTransactionEntryModel.setType(paymentTransactionType);
        paymentTransactionEntryModelList.add(paymentTransactionEntryModel);
        paymentTransactionModel.setEntries(paymentTransactionEntryModelList);
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);

        return orderModel;
    }

    private void setPaymentModeForPartialOperation(Boolean isAccepted) {
        standardPaymentModeModel.setPartialMaintenanceAllowed(isAccepted);
        orderModel.setPaymentMode(standardPaymentModeModel);
    }
}
