package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.controllers.PspaddonControllerConstants;
import com.psp.constants.PspConstants;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.payment.common.facades.order.PspCheckoutFacade;
import com.psp.payment.common.services.PspPaymentService;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commercefacades.user.UserFacade;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.Map;
import java.util.TreeMap;


/**
 * This callback controller will handle the response from PSP through the
 * customer browser. Any changes to the mapped URL should also be reflected in
 * the configuration properties psp.conf.<function>.url for e.g.
 * psp.conf.accept.url.
 */
@Controller
@RequestMapping(value = "/checkout/psp")
public class TokenisationCallbackController extends PaymentMethodCheckoutStepController {

    private static final Logger LOGGER = Logger.getLogger(TokenisationCallbackController.class);

    @Resource
    private PspPaymentService pspPaymentService;

    @Resource
    private UserFacade userFacade;

    @Resource
    private PspCheckoutFacade pspCheckoutFacade;

    /**
     * This method handles the accept callback from PSP. Any changes to the
     * mapped URL should also be reflected in the configuration properties
     * psp.conf.accept.url and psp.conf.exception.url.
     *
     * @param request the http request
     * @param model   the spring mvc model
     * @return The review step on successful transaction
     */
    @RequestMapping(value = "/response", method = {RequestMethod.POST, RequestMethod.GET})
    @RequireHardLogIn
    public String handleAcceptResponse(final HttpServletRequest request, final Model model, RedirectAttributes redirectAttributes) {

        String mainWindowParam = request.getParameter(PspConstants.PARAM_MAIN_WINDOW);

        if (StringUtils.isNotEmpty(mainWindowParam) && StringUtils.equals(mainWindowParam, PspConstants.TRUE)) {
            LOGGER.debug("Payment response received, processing it in the main window of the browser.");

            final Map<String, String> resultMap = getRequestParameterMap(request);
            final PspPaymentSubscriptionResultItem paymentResultData = pspPaymentService.completeHopCreatePaymentSubscription(resultMap);

            if (paymentResultData != null && paymentResultData.isSuccess() && paymentResultData.getStoredCard() != null) {
                final CCPaymentInfoData newPaymentSubscription = paymentResultData.getStoredCard();

                if (userFacade.getCCPaymentInfos(true).size() <= 1) {
                    userFacade.setDefaultPaymentInfo(newPaymentSubscription);
                }

                pspCheckoutFacade.setPaymentDetails(newPaymentSubscription.getId());
                pspCheckoutFacade.saveBillingAddressToPaymentInfo();
            } else if (paymentResultData != null) {
                LOGGER.error("Following error occurred during payment tokenisation: " + paymentResultData.getErrorCode());
                redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, paymentResultData.getErrorCode());

                return getCheckoutStep().currentStep();
            } else {
                LOGGER.error("Invalid response received from payment provider.");
                return getCheckoutStep().currentStep();
            }

            return getCheckoutStep().nextStep();
        }

        LOGGER.debug("Payment response received, popping out of the iframe");
        return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.ProcessPaymentFrame;
    }

    /**
     * Fetch all request parameters in a map.
     *
     * @param request the request object
     * @return map of all request parameters
     */
    protected Map<String, String> getRequestParameterMap(final HttpServletRequest request) {
        final Map<String, String> map = new TreeMap<>();
        final Enumeration<String> parameterNames = request.getParameterNames();
        LOGGER.info("***Tokenisation Response Parameters***");

        while (parameterNames.hasMoreElements()) {
            final String paramName = parameterNames.nextElement();
            final String paramValue = request.getParameter(paramName);

            if (StringUtils.containsIgnoreCase(paramName, "Alias.") && !StringUtils.containsIgnoreCase(paramName, "AliasId")) {
                LOGGER.info(paramName + " -> " + paramValue);
            }

            map.put(StringUtils.upperCase(paramName), paramValue);
        }

        return map;
    }
}
