package com.psp.addon.forms.validations;

import com.psp.addon.forms.PSPAddressForm;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import java.util.HashMap;
import java.util.Map;


/**
 * Validator for billing address forms. Enforces the order of validation.
 */
@Component("billingAddressValidator")
public class BillingAddressValidator implements Validator {
    private static final int MAX_FIELD_LENGTH = 255;
    private static final int MAX_POSTCODE_LENGTH = 10;

    @Override
    public boolean supports(final Class<?> aClass) {
        return PSPAddressForm.class.equals(aClass);
    }

    @Override
    public void validate(final Object object, final Errors errors) {
        final PSPAddressForm addressForm = (PSPAddressForm) object;
        validateStandardFields(addressForm, errors);
        validateCountrySpecificFields(addressForm, errors);
    }

    protected void validateStandardFields(final PSPAddressForm addressForm, final Errors errors) {
        validateStringField(addressForm.getBillTo_countryIso(), AddressField.COUNTRY, MAX_FIELD_LENGTH, errors);
        validateStringField(addressForm.getBillTo_firstName(), AddressField.FIRSTNAME, MAX_FIELD_LENGTH, errors);
        validateStringField(addressForm.getBillTo_lastName(), AddressField.LASTNAME, MAX_FIELD_LENGTH, errors);
        validateStringField(addressForm.getBillTo_line1(), AddressField.LINE1, MAX_FIELD_LENGTH, errors);
        validateStringField(addressForm.getBillTo_townCity(), AddressField.TOWN, MAX_FIELD_LENGTH, errors);
        validateStringField(addressForm.getBillTo_postcode(), AddressField.POSTCODE, MAX_POSTCODE_LENGTH, errors);
    }

    protected void validateCountrySpecificFields(final PSPAddressForm addressForm, final Errors errors) {
        final String isoCode = addressForm.getBillTo_countryIso();

        if (isoCode != null) {
            switch (CountryCode.lookup(isoCode)) {
                case CHINA:
                    validateStringField(addressForm.getBillTo_titleCode(), AddressField.TITLE, MAX_FIELD_LENGTH, errors);
                    validateFieldNotNull(addressForm.getBillTo_regionIso(), AddressField.REGION, errors);
                    break;
                case CANADA:
                    validateStringField(addressForm.getBillTo_titleCode(), AddressField.TITLE, MAX_FIELD_LENGTH, errors);
                    validateFieldNotNull(addressForm.getBillTo_regionIso(), AddressField.REGION, errors);
                    break;
                case USA:
                    validateStringField(addressForm.getBillTo_titleCode(), AddressField.TITLE, MAX_FIELD_LENGTH, errors);
                    validateFieldNotNull(addressForm.getBillTo_regionIso(), AddressField.REGION, errors);
                    break;
                case JAPAN:
                    validateFieldNotNull(addressForm.getBillTo_regionIso(), AddressField.REGION, errors);
                    validateStringField(addressForm.getBillTo_line2(), AddressField.LINE2, MAX_FIELD_LENGTH, errors);
                    break;
                default:
                    validateStringField(addressForm.getBillTo_titleCode(), AddressField.TITLE, MAX_FIELD_LENGTH, errors);
                    break;
            }
        }
    }

    protected static void validateStringField(final String addressField, final AddressField fieldType,
                                              final int maxFieldLength, final Errors errors) {
        if (StringUtils.isEmpty(addressField) || (StringUtils.length(addressField) > maxFieldLength)) {
            errors.rejectValue(fieldType.getFieldKey(), fieldType.getErrorKey());
        }
    }

    protected static void validateFieldNotNull(final String addressField, final AddressField fieldType,
                                               final Errors errors) {
        if (addressField == null) {
            errors.rejectValue(fieldType.getFieldKey(), fieldType.getErrorKey());
        }
    }

    protected enum CountryCode {
        USA("US"), CANADA("CA"), JAPAN("JP"), CHINA("CN"), BRITAIN("GB"), GERMANY("DE"), DEFAULT("");

        private String isoCode;

        private static Map<String, CountryCode> lookupMap = new HashMap<String, CountryCode>();

        static {
            for (final CountryCode code : CountryCode.values()) {
                lookupMap.put(code.getIsoCode(), code);
            }
        }

        private CountryCode(final String isoCodeStr) {
            this.isoCode = isoCodeStr;
        }

        public static CountryCode lookup(final String isoCodeStr) {
            CountryCode code = lookupMap.get(isoCodeStr);
            if (code == null) {
                code = DEFAULT;
            }
            return code;
        }

        public String getIsoCode() {
            return isoCode;
        }
    }

    protected enum AddressField {
        TITLE("billTo_titleCode", "address.title.invalid"), FIRSTNAME("billTo_firstName", "address.firstName.invalid"),
        LASTNAME("billTo_lastName", "address.lastName.invalid"), LINE1("billTo_line1", "address.line1.invalid"),
        LINE2("billTo_line2", "address.line2.invalid"), TOWN("billTo_townCity", "address.townCity.invalid"),
        POSTCODE("billTo_postcode", "address.postcode.invalid"), REGION("billTo_regionIso", "address.regionIso.invalid"),
        COUNTRY("billTo_postcode", "address.country.invalid");

        private String fieldKey;
        private String errorKey;

        private AddressField(final String fieldKey, final String errorKey) {
            this.fieldKey = fieldKey;
            this.errorKey = errorKey;
        }

        public String getFieldKey() {
            return fieldKey;
        }

        public String getErrorKey() {
            return errorKey;
        }
    }
}
